from lib.common.evolution.tree_operate import get_sons
from Bio import Phylo
from config import badirate_path
from lib.common.os import mkdir, rmdir, cmd_run, multiprocess_running
from lib.common.util import logging_init
import copy
import os
import re
from io import StringIO
import math


def F_index(gene_family_size_tuple):
    S = len(gene_family_size_tuple)
    a = (S * S - 2 * S) / 2
    N = sum(gene_family_size_tuple)

    F = [(math.log2(a*c/N + 0.5) + 1) / (math.log2(a + 0.5) + 1)
         for c in gene_family_size_tuple]

    return F


def run_badirate_one(ctl_file, badirate_path):
    cmd_run("perl %s %s" % (badirate_path, ctl_file), silence=True)


def run_badirate(ctl_list, output_list, badirate_path, just_load=False):
    if just_load:
        output_file_dict = {}

        for output_file in output_list:
            output_file_dict[output_file] = 0
            if os.path.exists(output_file):
                output_file_dict[output_file] = os.path.getsize(output_file)
    else:
        # list_run=[ctl_file1,ctl_file2,..]
        for file_tmp in output_list:
            if os.path.exists(file_tmp):
                os.remove(file_tmp)

        for ctl_file in ctl_list:
            run_badirate_one(ctl_file, badirate_path)

        output_file_dict = {}

        for output_file in output_list:
            output_file_dict[output_file] = 0
            if os.path.exists(output_file):
                output_file_dict[output_file] = os.path.getsize(output_file)

    return output_file_dict


def get_K(bmodel_string):
    return 2*(len(re.findall(r'_', bmodel_string))+1)


def two_better_than_one_significance(k1, Likelihood1, k2, Likelihood2):
    AIC1 = 2*float(k1)-2*float(Likelihood1)
    AIC2 = 2*float(k2)-2*float(Likelihood2)

    if max([AIC1, AIC2])-min([AIC1, AIC2]) > 2:
        if AIC1 > AIC2:
            return 1, AIC1, AIC2
        elif AIC1 < AIC2:
            return -1, AIC1, AIC2
    else:
        return 0, AIC1, AIC2


def get_branch_name(badirate_tree, size_file, badirate_path):
    """
    perl ~/Program/badirate/badirate-master/BadiRate.pl -sizefile 1 -print_ids -treefile species_tree.tre > species_tree.badirate.tre
    """

    cmd_string = "perl %s -sizefile %s -print_ids -treefile %s" % (
        badirate_path, size_file, badirate_tree)
    flag, output, error = cmd_run(cmd_string, silence=True)

    labeled_tree_string = output.split('\n')[1]
    labeled_tree = StringIO(labeled_tree_string)

    tree = Phylo.read(labeled_tree, 'newick')

    all_branch = []
    
    for clade in tree.find_clades(order='level'):
        if clade.is_terminal():
            continue
        else:
            for son in get_sons(clade):
                clade_name = clade.confidence
                if son.is_terminal():
                    son_name = son.name.split("_")[-1]
                else:
                    son_name = son.confidence

                all_branch.append("%s->%s" % (clade_name, son_name))

    return all_branch, labeled_tree_string


def make_bmodel_string(all_branch, known="", new_ratio=0, GR=0, FR=0):
    """
    all_branch = ["17->15","15->11","11->9","9->3","3->1","3->2","9->8","8->6","6->4","6->5","8->7","11->10","15->14","14->12","14->13","17->16"]
    """
    all_string = {}

    if GR == 1:
        output = ""
        for i in all_branch:
            output = output+i+":"
        output = output.rstrip(":")
        return output

    if FR == 1:
        output = ""
        for i in all_branch:
            output = output+i+"_"
        output = output.rstrip("_")
        return output

    if known == "":
        head_string = known
        remain_branch = copy.deepcopy(all_branch)
    else:
        if new_ratio == 1:
            head_string = known+"_"
        elif new_ratio == 0:
            head_string = known+":"
        known_branchs = []
        ratios = known.split("_")
        for ratio in ratios:
            branchs = ratio.split(":")
            for branch in branchs:
                known_branchs.append(branch)

        remain_branch = copy.deepcopy(all_branch)
        for branch in known_branchs:
            remain_branch.remove(branch)

    output = {}
    for branch in remain_branch:
        output_string = head_string+branch+"_"
        for branch_other in remain_branch:
            if branch == branch_other:
                continue
            output_string = output_string + branch_other + ":"
        output_string = output_string.rstrip(":")
        output[branch] = output_string
    return head_string, output, new_ratio


def make_control_file(bmodel_string, size_file, treefile, work_dir, num, just_load=False):

    clt_0_file = "%s/%s.0.ctl" % (work_dir, str(num))
    out_0_file = "%s/%s.0.out" % (work_dir, str(num))

    if not just_load:

        with open(clt_0_file, 'w') as f:
            f.write(
                """
root_dist = 1
sizefile = %s
treefile = %s
n_max_int = 10
priorfile = 0
outlier = 0
seed = 587347092
unobs = 1
rmodel = GD
ep = ML
help = 0
out = %s
anc = 1
version = 0
print_ids = 0
bmodel = %s
start_val = 0
family = 0
    """ % (size_file, treefile, out_0_file, bmodel_string))

    clt_1_file = "%s/%s.1.ctl" % (work_dir, str(num))
    out_1_file = "%s/%s.1.out" % (work_dir, str(num))

    if not just_load:

        with open(clt_1_file, 'w') as f:
            f.write(
                """
root_dist = 1
sizefile = %s
treefile = %s
n_max_int = 10
priorfile = 0
outlier = 0
seed = 1
unobs = 1
rmodel = GD
ep = ML
help = 0
out = %s
anc = 1
version = 0
print_ids = 0
bmodel = %s
start_val = 1
family = 0
    """ % (size_file, treefile, out_1_file, bmodel_string))

    return [[clt_0_file, out_0_file], [clt_1_file, out_1_file]]


def get_ancestral_size(file_name):
    F1 = open(file_name)
    # print file_name
    all_text = F1.read()
    info = all_text.split('--------------------\n')

    label_tree_string = info[0].split("\n")[1]

    for i in info[2].split('\n'):
        match = re.match(r'\t\tTotal Ancestral Size\t(\S+)', i)
        if match:
            ancestral_size_string = match.group(1)

    
    ancestral_size_tree = Phylo.read(StringIO(ancestral_size_string), 'newick')

    ancestral_size_tuple = []
    for i in ancestral_size_tree.find_clades(order='preorder'):
        if i.is_terminal():
            node_num = int(i.name.split("_")[-1])
        else:
            node_num = int(i.confidence)
        ancestral_size_tuple.append(node_num)
    ancestral_size_tuple = tuple(ancestral_size_tuple)

    label_tree_tree = Phylo.read(StringIO(label_tree_string), 'newick')

    label_tuple = []
    for i in label_tree_tree.find_clades(order='preorder'):
        if i.is_terminal():
            node_num = str(i.name.split("_")[-1])
        else:
            node_num = str(i.confidence)
        label_tuple.append(node_num)
    label_tuple = tuple(label_tuple)

    ancestral_size_dict = {}

    for i in range(len(label_tuple)):
        ancestral_size_dict[label_tuple[i]] = int(ancestral_size_tuple[i])

    F1.close()

    return ancestral_size_dict



def detect_pure_gain_and_loss(ancestral_size_dict, all_branch):
    gain_list = []
    loss_list = []
    for branch in all_branch:
        node_f, node_t = branch.split('->')
        if ancestral_size_dict[node_f] == 0 and ancestral_size_dict[node_t] > 0:
            gain_list.append(branch)
        elif ancestral_size_dict[node_f] > 0 and ancestral_size_dict[node_t] == 0:
            loss_list.append(branch)

    return gain_list, loss_list


def badirate_output_parse(file_name):
    F1 = open(file_name)
    # print file_name
    all_text = F1.read()
    info = all_text.split('--------------------\n')
    # print info
    while '' in info:
        info.remove('')
    INTERNAL_ID_TREE = info[0]
    INPUT = info[1]
    OUTPUT = info[2]
    info_output = OUTPUT.split('\n')
    for i in info_output:
        match = re.match(r'\t\t#Likelihood: (\S+)', i)
        if match:
            Likelihood = match.group(1)
    # print Likelihood
    info_output = OUTPUT.split('##')
    flag = 0
    mini_dict = {}
    for i in info_output:
        match = re.match(r'Minimum number of gains and losses per branch', i)
        if match:
            min_num = i.split('\n')
            for line in min_num:
                match2 = re.match(r'\t\t(\d+\->\d+)\t(\d+)\t(\d+)', line)
                if match2:
                    mini_dict[match2.group(1)] = [int(
                        match2.group(2)), int(match2.group(3))]

    F1.close()

    return INPUT, mini_dict, float(Likelihood)


def get_best_start_value(input_list):
    [like1, like2] = input_list
    if like1 == '-inf':
        like1 = -99999999999999999999
    else:
        like1 = float(like1)

    if like2 == '-inf':
        like2 = -99999999999999999999
    else:
        like2 = float(like2)

    temp_list = [like1, like2]
    best = max(temp_list)
    if best == -99999999999999999999:
        return "-inf"
    else:
        return temp_list.index(best)


def main_pipeline(tag, size_file, species_tree, work_dir, badirate_path, label_tree_path=None, keep_tmp_dir=False, just_load=False):
    log_file = work_dir + "/log"

    if just_load:

        all_branch, label_tree = get_branch_name(
            species_tree, size_file, badirate_path)


        FR_string = make_bmodel_string(all_branch, FR=1)
        out_list = make_control_file(
            FR_string, size_file, species_tree, work_dir, "FR", just_load)

        a = [out_list[0][0], out_list[1][0]]
        b = [out_list[0][1], out_list[1][1]]

        output_file_dict = run_badirate(a, b, badirate_path, just_load)

        output_file_list = list(output_file_dict)

        like_FR = [badirate_output_parse(file)[2] for file in output_file_list]

        best_like_FR_index = get_best_start_value(like_FR)

        if best_like_FR_index == "-inf":
            raise
        else:
            best_like_FR = like_FR[best_like_FR_index]

        FR_output_file = output_file_list[best_like_FR_index]

        ancestral_size = get_ancestral_size(FR_output_file)
        pure_gain, pure_loss = detect_pure_gain_and_loss(ancestral_size, all_branch)

        INPUT, mini_dict, Likelihood = badirate_output_parse(FR_output_file)
        back_branch = []
        test_branch = []
        for i in mini_dict:
            gains, losses = mini_dict[i]
            if gains == losses and gains == 0:
                back_branch.append(i)
            else:
                test_branch.append(i)

        # not branch changed
        if len(back_branch) == len(all_branch):
            # return [tag, [], [], Likelihood]
            print([tag, [], [], Likelihood])
            raise

        like_FR = sorted(like_FR, reverse=True)

        eFR_model_string = ""
        for i in back_branch:
            eFR_model_string = eFR_model_string+i+":"
        eFR_model_string = eFR_model_string.rstrip(":")
        back_branch_string = eFR_model_string
        eFR_model_string = eFR_model_string+"_"

        for i in all_branch:
            if i in back_branch:
                continue
            eFR_model_string = eFR_model_string+i+"_"
        eFR_model_string = eFR_model_string.rstrip("_")

        out_list = make_control_file(
            eFR_model_string, size_file, species_tree, work_dir, "eFR", just_load)

        a = [out_list[0][0], out_list[1][0]]
        b = [out_list[0][1], out_list[1][1]]

        output_file_dict = run_badirate(a, b, badirate_path, just_load)

        output_file_list = list(output_file_dict)

        like_eFR = [badirate_output_parse(file)[2] for file in output_file_list]

        best_like_eFR_index = get_best_start_value(like_eFR)
        if best_like_eFR_index == "-inf":
            raise
        else:
            best_like_eFR = like_eFR[best_like_eFR_index]

        eFR_output_file = output_file_list[get_best_start_value(like_eFR)]

        INPUT, mini_dict, Likelihood = badirate_output_parse(eFR_output_file)

        output_dict = {}
        output_dict["family_id"] = tag
        output_dict["FR"] = FR_output_file
        output_dict["eFR"] = {}
        output_dict["tsv_file"] = size_file
        output_dict["eFR"]['Likelihood'] = Likelihood
        output_dict["eFR"]['K'] = get_K(eFR_model_string)
        output_dict["eFR"]['output_file'] = eFR_output_file

        # begin test

        good_branch = []
        for branch_now in test_branch:

            num = re.sub("->", '_', branch_now)
            output_dict[num] = {}
            output_dict[num]['branch'] = branch_now
            test_branch_string = back_branch_string+":"+branch_now+"_"
            for j in test_branch:
                if not j == branch_now:
                    test_branch_string = test_branch_string+j+"_"
            test_branch_string = test_branch_string.rstrip("_")

            out_list = make_control_file(
                test_branch_string, size_file, species_tree, work_dir, num, just_load)

            a = [out_list[0][0], out_list[1][0]]
            b = [out_list[0][1], out_list[1][1]]

            output_file_dict = run_badirate(a, b, badirate_path, just_load)

            like_now = [(badirate_output_parse(file[1])[2], file)
                        for file in out_list]
            like_now = sorted(like_now, reverse=True)
            best_eFR = like_now[0]

            if best_eFR[0] == '-inf':
                output_dict[num]['out_file'] = best_eFR[1][1]
                output_dict[num]['Likelihood'] = '-99999999999999999999'
                output_dict[num]['bmodel_string'] = test_branch_string
                output_dict[num]['K'] = get_K(test_branch_string)
            else:
                output_dict[num]['out_file'] = best_eFR[1][1]
                output_dict[num]['Likelihood'] = best_eFR[0]
                output_dict[num]['bmodel_string'] = test_branch_string
                output_dict[num]['K'] = get_K(test_branch_string)

            flag, AIC1, AIC2 = two_better_than_one_significance(
                output_dict[num]['K'], output_dict[num]['Likelihood'], output_dict["eFR"]['K'], output_dict["eFR"]['Likelihood'])

            if flag == 1:
                output_dict[num]['significative'] = 1
                good_branch.append(branch_now)
            else:
                output_dict[num]['significative'] = 0

    else:

        mkdir(work_dir, False)

        logger = logging_init("badirate_exp_con", log_file)
        logger.info("Build work dir")

        all_branch, label_tree = get_branch_name(
            species_tree, size_file, badirate_path)

        if not label_tree_path is None:
            with open(label_tree_path, 'w') as f:
                f.write(label_tree)

        # cal free branch model
        logger.info("cal free branch model")

        FR_string = make_bmodel_string(all_branch, FR=1)
        out_list = make_control_file(
            FR_string, size_file, species_tree, work_dir, "FR")

        a = [out_list[0][0], out_list[1][0]]
        b = [out_list[0][1], out_list[1][1]]

        output_file_dict = run_badirate(a, b, badirate_path)
        logger.info("cal free branch model, done!")

        output_file_list = list(output_file_dict)

        like_FR = [badirate_output_parse(file)[2] for file in output_file_list]

        best_like_FR_index = get_best_start_value(like_FR)

        if best_like_FR_index == "-inf":
            logger.info("%s can't get information from free model, failed!" % tag)
            # return None
            raise
        else:
            best_like_FR = like_FR[best_like_FR_index]

        FR_output_file = output_file_list[best_like_FR_index]

        ancestral_size = get_ancestral_size(FR_output_file)
        pure_gain, pure_loss = detect_pure_gain_and_loss(ancestral_size, all_branch)

        INPUT, mini_dict, Likelihood = badirate_output_parse(FR_output_file)
        back_branch = []
        test_branch = []
        for i in mini_dict:
            gains, losses = mini_dict[i]
            if gains == losses and gains == 0:
                back_branch.append(i)
            else:
                test_branch.append(i)

        logger.info("free branch model get likelihood: %.5f, and %d branch have change" % (
            Likelihood, len(all_branch) - len(back_branch)))
        # not branch changed
        if len(back_branch) == len(all_branch):
            logger.info("No exp & con")
            # return [tag, [], [], Likelihood]
            print([tag, [], [], Likelihood])
            raise

        like_FR = sorted(like_FR, reverse=True)

        logger.info("cal Null hypothesis branch model")

        eFR_model_string = ""
        for i in back_branch:
            eFR_model_string = eFR_model_string+i+":"
        eFR_model_string = eFR_model_string.rstrip(":")
        back_branch_string = eFR_model_string
        eFR_model_string = eFR_model_string+"_"

        for i in all_branch:
            if i in back_branch:
                continue
            eFR_model_string = eFR_model_string+i+"_"
        eFR_model_string = eFR_model_string.rstrip("_")

        logger.info("get eFR model string: %s" % eFR_model_string)
        out_list = make_control_file(
            eFR_model_string, size_file, species_tree, work_dir, "eFR")

        a = [out_list[0][0], out_list[1][0]]
        b = [out_list[0][1], out_list[1][1]]

        output_file_dict = run_badirate(a, b, badirate_path)
        logger.info("cal Null hypothesis branch model, done!")

        output_file_list = list(output_file_dict)

        like_eFR = [badirate_output_parse(file)[2] for file in output_file_list]

        best_like_eFR_index = get_best_start_value(like_eFR)
        if best_like_eFR_index == "-inf":
            logger.info(
                "%s can't get information from null hypothesis free model, failed!" % tag)
            # return None
            raise
        else:
            best_like_eFR = like_eFR[best_like_eFR_index]

        eFR_output_file = output_file_list[get_best_start_value(like_eFR)]

        INPUT, mini_dict, Likelihood = badirate_output_parse(eFR_output_file)

        output_dict = {}
        output_dict["family_id"] = tag
        output_dict["FR"] = FR_output_file
        output_dict["eFR"] = {}
        output_dict["tsv_file"] = size_file
        output_dict["eFR"]['Likelihood'] = Likelihood
        output_dict["eFR"]['K'] = get_K(eFR_model_string)
        output_dict["eFR"]['output_file'] = eFR_output_file

        logger.info("null hypothesis branch model get likelihood: %.5f, K is %d" % (
            Likelihood, output_dict["eFR"]['K']))

        # begin test

        good_branch = []
        for branch_now in test_branch:

            logger.info("test %s %s now:" % (branch_now, mini_dict[branch_now]))

            logger.info("cal test branch model")
            num = re.sub("->", '_', branch_now)
            output_dict[num] = {}
            output_dict[num]['branch'] = branch_now
            test_branch_string = back_branch_string+":"+branch_now+"_"
            for j in test_branch:
                if not j == branch_now:
                    test_branch_string = test_branch_string+j+"_"
            test_branch_string = test_branch_string.rstrip("_")

            logger.info("get test branch model string: %s" % test_branch_string)

            out_list = make_control_file(
                test_branch_string, size_file, species_tree, work_dir, num)

            a = [out_list[0][0], out_list[1][0]]
            b = [out_list[0][1], out_list[1][1]]

            output_file_dict = run_badirate(a, b, badirate_path)
            logger.info("cal test branch model, done!")

            like_now = [(badirate_output_parse(file[1])[2], file)
                        for file in out_list]
            like_now = sorted(like_now, reverse=True)
            best_eFR = like_now[0]

            if best_eFR[0] == '-inf':
                output_dict[num]['out_file'] = best_eFR[1][1]
                output_dict[num]['Likelihood'] = '-99999999999999999999'
                output_dict[num]['bmodel_string'] = test_branch_string
                output_dict[num]['K'] = get_K(test_branch_string)
            else:
                output_dict[num]['out_file'] = best_eFR[1][1]
                output_dict[num]['Likelihood'] = best_eFR[0]
                output_dict[num]['bmodel_string'] = test_branch_string
                output_dict[num]['K'] = get_K(test_branch_string)

            logger.info("comp %s (%.5f, %d) and eFR (%.5f, %d)" % (
                branch_now, output_dict[num]['Likelihood'], output_dict[num]['K'], output_dict["eFR"]['Likelihood'], output_dict["eFR"]['K']))

            flag, AIC1, AIC2 = two_better_than_one_significance(
                output_dict[num]['K'], output_dict[num]['Likelihood'], output_dict["eFR"]['K'], output_dict["eFR"]['Likelihood'])

            logger.info("AIC: %.5f vs %.5f flag: %d" % (AIC1, AIC2, flag))

            if flag == 1:
                output_dict[num]['significative'] = 1
                good_branch.append(branch_now)
            else:
                output_dict[num]['significative'] = 0

    up_down = {}
    up_down['up'] = []
    up_down['down'] = []
    for i in good_branch:
        if mini_dict[i][0] > 0:
            up_down['up'].append(i)
        elif mini_dict[i][1] > 0:
            up_down['down'].append(i)

    up_down['up'] = list(set(up_down['up'] + pure_gain))
    up_down['down'] = list(set(up_down['down'] + pure_loss))

    up_down['up'] = list(set(up_down['up']) - set(pure_gain))
    up_down['down'] = list(set(up_down['down']) - set(pure_loss))

    printer = ""
    for i in up_down['up']:
        printer = printer+i+","
    printer = printer.rstrip(",")
    printer = printer+"\t"
    for i in up_down['down']:
        printer = printer+i+","
    printer = printer.rstrip(",")
    printer = printer + "\t" + str(output_dict["eFR"]['Likelihood'])
    printer = tag + "\t" + printer

    print("Tag\tGain\tLoss\tExpansion\tContraction\tLikelihood")
    print(tag, pure_gain, pure_loss, up_down['up'], up_down['down'],
        output_dict["eFR"]['Likelihood'])

    if keep_tmp_dir:
        print("temp_dir is : %s" % work_dir)
    else:
        rmdir(work_dir)

    return tag, pure_gain, pure_loss, up_down['up'], up_down['down'], output_dict["eFR"]['Likelihood'], label_tree


def EasyBadiRate_main():
    pass


if __name__ == "__main__":

    species_tree = '/lustre/home/xuyuxing/Work/Orobanchaceae/gene_family/hcluster/test/tre2'
    label_tree_path = '/lustre/home/xuyuxing/Work/Orobanchaceae/gene_family/hcluster/test/label.tre'
    size_file = '/lustre/home/xuyuxing/Work/Orobanchaceae/gene_family/hcluster/test/tsv'
    work_dir = '/lustre/home/xuyuxing/Work/Orobanchaceae/gene_family/hcluster/test/out'
    log_file = work_dir + "/log"
    tag = "gene1"

    output = main_pipeline(tag, size_file, species_tree,
                           work_dir, badirate_path, label_tree_path, True)

    if output:
        tag, up_list, down_list, ll, label_tree = output


    tag="C1194"
    size_file="/lustre/home/xuyuxing/Work/Orobanchaceae/publish/3.gene_family/2.expansion_contract/GeneFamilyStat/C1194/size_file"
    species_tree="/lustre/home/xuyuxing/Work/Orobanchaceae/publish/3.gene_family/2.expansion_contract/species_time.tre"
    work_dir="/lustre/home/xuyuxing/Work/Orobanchaceae/publish/3.gene_family/2.expansion_contract/GeneFamilyStat/C1194/run"
    badirate_path="/lustre/home/xuyuxing/Program/badirate/badirate-master/BadiRate.pl"
    label_tree_path=None
    keep_tmp_dir=True
    just_load=True

    output = main_pipeline(tag, size_file, species_tree, work_dir, badirate_path, label_tree_path, keep_tmp_dir, just_load)


import time
import uuid
import pandas as pd
from itertools import combinations
import networkx as nx
from interlap import InterLap
from collections import Counter
from lib.common.genome.seq_base import read_fasta_by_faidx
import lib.common.sqlite_command as sc
from lib.common.os import mkdir, multiprocess_running
from lib.common.genome.genome_feature2 import read_gff_file, get_gf_db_meta_dict, gf_info_retrieval, write_gff_file
from lib.xuyuxing.math.set_operating import interval_length_sum
from lib.common.math.interval import section, merge_intervals, interval_minus_set, overlap_between_interval_set, \
    group_by_intervals_with_overlap_threshold
from api.xuyuxing.comp_genome.orthofinder import OG_tsv_file_parse
from lib.xuyuxing.math.stats import get_threshold
from lib.common.util import printer_list
from lib.common.math.bin_and_window import split_sequence_to_bins


def extract_one_evidences(id_tmp, gf_name, type_tmp, contig_name, start, end, strand, daughter, qualifiers, gws_db, OG_support, taxon_id, table_name):
    gf = gf_info_retrieval((id_tmp, gf_name, type_tmp, contig_name, start, end, strand, daughter, qualifiers), 0, 'A', gws_db)

    mRNA_gf = gf.sub_features[0]
    qualifiers = mRNA_gf.qualifiers

    # query coverage
    coverage = min((int(qualifiers['Target_End'][0]) - int(
        qualifiers['Target_Start'][0]) + 1) / int(qualifiers['Target_Length'][0]), 1.00)

    # aln aa
    aln_aa_len = int(
        qualifiers['Target_End'][0]) - int(qualifiers['Target_Start'][0]) + 1

    # identity
    identity = int(qualifiers['idt_len']) / \
        int(qualifiers['aln_len'])

    q_id = gf_name.split("_")[0]

    gf.evidence_indicator = {
        "query_id": q_id,
        "query_coverage": coverage,
        "aln_aa_len": aln_aa_len,
        "identity": identity,
        "query_length": int(qualifiers['Target_Length'][0]),
        "score": max(0.1, float(gf.sub_features[0].qualifiers['score'][0])),
        "OG_support": OG_support
    }

    gf.score_dict = {
        "score": max(0.1, float(gf.sub_features[0].qualifiers['score'][0]))
    }

    gf.db_path = "%s.%s.%s" % (taxon_id, table_name, id_tmp)

    return gf


def extract_evidences(contig, strand, ref_WPGmapper_dict, seq_support_dict, given_range, num_thread):

    # get all useable evidence on the contig(strand)

    evidence_dict = {}
    args_list = []
    args_id_list = []

    for taxon_id in ref_WPGmapper_dict:
        # print(taxon_id)

        gws_db = ref_WPGmapper_dict[taxon_id]['genewise']
        meta_dict = get_gf_db_meta_dict(gws_db)

        # read mRNA info

        for table_name in meta_dict['gene'][0]:
            # print(table_name)
            if given_range is None:
                sql_cmd_string = 'SELECT * FROM %s WHERE contig_name = \"%s\" AND strand = \"%s\"' % (
                    table_name, contig, strand)
            else:
                sql_cmd_string = 'SELECT * FROM %s WHERE contig_name = \"%s\" AND strand = \"%s\" AND start <= %d AND end >= %d' % (
                    table_name, contig, strand, max(given_range), min(given_range))

            data_list = sc.sqlite_execute(sql_cmd_string, gws_db)
            for gf_db_info_tuple in data_list:
                id_tmp, gf_name, type_tmp, contig_name, start, end, strand, daughter, qualifiers = gf_db_info_tuple

                q_id = gf_name.split("_")[0]
                if q_id in seq_support_dict:
                    OG_support = seq_support_dict[q_id]
                else:
                    OG_support = (0, 0)

                args_list.append((id_tmp, gf_name, type_tmp, contig_name, start, end, strand, daughter, qualifiers, gws_db, OG_support, taxon_id, table_name))
                args_id_list.append(gf_name)

    ev_output = multiprocess_running(extract_one_evidences, args_list, num_thread, silence=True, args_id_list=args_id_list)

    for i in ev_output:
        evidence_dict[i] = ev_output[i]['output']
    
    return evidence_dict



import os
import re
import random
import pickle
from BCBio import GFF
from pyfaidx import Fasta
from interlap import InterLap
from Bio.Seq import Seq, translate
from Bio.SeqRecord import SeqRecord
from Bio.SeqFeature import SeqFeature, FeatureLocation
from lib.common.util import logging_init
from lib.common.os import mkdir, cmd_run, multiprocess_running
from lib.common.genome.seq_base import read_fasta_by_faidx
from lib.common.genome.genome_feature2 import cds_judgment, ChrLoci, GenomeFeature, write_gff_file, cluster_of_chr_loci
from lib.xuyuxing.seq.genome_feature import GenomeFeature as old_GenomeFeature
from lib.common.math.interval import section, merge_intervals, interval_minus_set
# from lib.xuyuxing.seq.genome_feature import ChrLoci, GenomeFeature, cluster_of_chr_loci, write_GenomeFeature_to_GFF
from api.xuyuxing.genome.genblasta import genblasta_run, genblasta_to_genewise, get_HPR_from_genewise_results, gene_to_aa, \
    gene_to_cds, query_gene_to_speci
from api.common.genome.blast import blastn_pair_running


def cDNA_coord_shift(give_site, direction, sub_feature_list):
    """
    give_site = 100
    direction = 'cDNA_to_Genome'
    sub_feature_list = mRNA.sub_features
    """

    strand = sub_feature_list[0].strand

    if strand == 1 or strand == "+":
        GC = sorted([(exon.location.start + 1, exon.location.end + 0) for exon in sub_feature_list if
                     exon.type == 'exon' or exon.type == 'match_part'])
    else:
        GC = sorted([(exon.location.end + 0, exon.location.start + 1) for exon in sub_feature_list if
                     exon.type == 'exon' or exon.type == 'match_part'], reverse=True)

    start = 0
    CC = []
    for i in GC:
        i_len = abs(i[1] - i[0]) + 1
        CC.append((start + 1, start + i_len))
        start = start + i_len

    if direction == 'cDNA_to_Genome':
        block_index = None
        for i in CC:
            if min(i) <= give_site <= max(i):
                block_index = CC.index(i)

        if block_index is None:
            raise ValueError("give_site is out of gene range")

        dela_len = give_site - CC[block_index][0]

        if strand == 1 or strand == "+":
            return GC[block_index][0] + dela_len
        else:
            return GC[block_index][0] - dela_len

    elif direction == 'Genome_to_cDNA':
        block_index = None
        for i in GC:
            if min(i) <= give_site <= max(i):
                block_index = GC.index(i)

        if block_index is None:
            raise ValueError("give_site is out of gene range")

        if strand == 1 or strand == "+":
            dela_len = give_site - GC[block_index][0]
        else:
            dela_len = GC[block_index][0] - give_site

        return CC[block_index][0] + dela_len


def cDNA_feature_coord(mRNA):
    output_list = []
    for ft in mRNA.sub_features:
        if ft.type == 'exon':
            continue

        ft_start = ft.location.start + 1
        ft_end = ft.location.end

        ft_cDNA_start_tmp = cDNA_coord_shift(
            ft_start, "Genome_to_cDNA", mRNA.sub_features)
        ft_cDNA_end_tmp = cDNA_coord_shift(
            ft_end, "Genome_to_cDNA", mRNA.sub_features)

        ft_cDNA_start = min(ft_cDNA_start_tmp, ft_cDNA_end_tmp)
        ft_cDNA_end = max(ft_cDNA_start_tmp, ft_cDNA_end_tmp)

        output_list.append((ft.type, (ft_cDNA_start, ft_cDNA_end)))

    return output_list


def extract_give_feature_seq(give_type, sub_features_list, chr_id, strand, genome_file):
    get_seq = ''

    if strand == 1 or strand == "+":
        feature_list = sorted([ft for ft in sub_features_list if ft.type == give_type],
                              key=lambda x: int(x.location.start), reverse=False)
    else:
        feature_list = sorted([ft for ft in sub_features_list if ft.type == give_type],
                              key=lambda x: int(x.location.start), reverse=True)

    for ft in feature_list:
        ft = old_GenomeFeature(name=ft.id, chr_id=chr_id,
                           type='CDS', bio_location=ft.location)
        get_seq += ft.get_sequence(genome_file).seq

    return get_seq


def split_blat_gff(blat_gff_file, split_prefix, target_per_file):
    file_num = 0
    file_list = []
    with open(blat_gff_file, 'r') as f:
        last_t_id = ''
        save_target = 0
        for each_line in f:
            mbj = re.findall('Target=(\S+)', each_line)
            if len(mbj) == 0:
                continue
            t_id = mbj[0]
            if t_id != last_t_id:
                save_target += 1
                if save_target == target_per_file or file_num == 0:
                    if file_num > 0:
                        F1.close()
                    F1 = open("%s_%d.gff" % (split_prefix, file_num), 'w')
                    file_list.append("%s_%d.gff" % (split_prefix, file_num))
                    file_num += 1
                    save_target = 0
                last_t_id = t_id
            F1.write(each_line)
    return file_list


def split_and_load_blat_hit(raw_blat_gff_file, feature_type, split_dir, threads=80, target_per_file=1):
    mkdir(split_dir, False)

    split_file_list = split_blat_gff(raw_blat_gff_file, split_dir+"/split", target_per_file=target_per_file)
    # print('finished split')

    args_list = []
    for split_file in split_file_list:
        args_list.append((split_file, feature_type))

    mlt_out = multiprocess_running(blat_hit_load, args_list, threads, silence=False)

    dict_tmp = {}
    for i in mlt_out:
        for j in mlt_out[i]['output']:
            dict_tmp[j] = mlt_out[i]['output'][j]

    return dict_tmp


def blat_hit_load(blat_gff_file, feature_type):
    
    dict_tmp = {}

    with open(blat_gff_file, 'r') as in_handle:
        for rec in GFF.parse(in_handle):

            for feature in rec.features:
                if feature.type != 'inferred_parent':
                    feature_tmp = SeqFeature(id=feature.qualifiers['Parent'][0], location=feature.location,
                                             type='inferred_parent', strand=feature.strand,
                                             qualifiers={'ID': feature.qualifiers['Parent']})
                    feature_tmp.sub_features = [feature]
                    feature = feature_tmp

                chr_loci_tmp = ChrLoci(chr_id=rec.id, strand=feature.strand,
                                       start=feature.location.start + 1,
                                       end=feature.location.end)
                chr_loci_tmp.query_type = feature_type

                feature_xyx = old_GenomeFeature(feature.id, type=feature.type, chr_loci=chr_loci_tmp,
                                            sub_feature=feature.sub_features)
                feature_xyx.target = feature.sub_features[0].qualifiers['Target'][0].split()[
                    0]
                feature_xyx.score = float(
                    feature.sub_features[0].qualifiers['score'][0])
                feature_xyx.query_type = feature_type

                chr_loci_feature = []
                q_site_range = []
                for sub_feature_tmp in feature_xyx.sub_feature:
                    chr_loci_tmp = ChrLoci(chr_id=rec.id, strand=sub_feature_tmp.strand,
                                           start=sub_feature_tmp.location.start + 1,
                                           end=sub_feature_tmp.location.end)
                    chr_loci_tmp.type = sub_feature_tmp.type

                    target_info = sub_feature_tmp.qualifiers['Target'][0].split(
                    )
                    query_range = [
                        int(target_info[1]), int(target_info[2])]
                    chr_loci_tmp.target = target_info[0]
                    chr_loci_tmp.q_start = min(query_range)
                    chr_loci_tmp.q_end = max(query_range)
                    chr_loci_tmp.location = sub_feature_tmp.location
                    q_site_range.append(chr_loci_tmp.q_start)
                    q_site_range.append(chr_loci_tmp.q_end)
                    chr_loci_tmp.id = sub_feature_tmp.id

                    chr_loci_feature.append(chr_loci_tmp)

                feature_xyx.q_start = min(q_site_range)
                feature_xyx.q_end = max(q_site_range)

                feature_xyx.sub_feature = chr_loci_feature

                if feature_xyx.target not in dict_tmp:
                    dict_tmp[feature_xyx.target] = []

                dict_tmp[feature_xyx.target].append(feature_xyx)

    for i in dict_tmp:
        dict_tmp[i] = sorted(
            dict_tmp[i], key=lambda x: x.score, reverse=True)

    # print(blat_gff_file + ".OK")
    return dict_tmp


def no_gap_index(seq, gap_chr='-'):
    """0-base"""
    ind_list = []
    for i in range(len(seq)):
        c = seq[i]
        if i > 0:
            lc = seq[i - 1]

        # block start
        if i == 0 and c != gap_chr:
            blocking_start = 0
        if i != 0 and c != gap_chr and lc == gap_chr:
            blocking_start = i

        # block end
        if i != 0 and c == gap_chr and lc != gap_chr:
            blocking_end = i - 1
            ind_list.append((blocking_start, blocking_end))
        if i + 1 == len(seq):
            blocking_end = i
            ind_list.append((blocking_start, blocking_end))

    seq_index_interlap = InterLap()

    no_gap_end_index = -1

    for i in ind_list:
        seq_index_interlap.add(
            (no_gap_end_index + 1, i[1] - i[0] + 1 + no_gap_end_index, i))
        no_gap_end_index = i[1] - i[0] + 1 + no_gap_end_index

    return seq_index_interlap


def blast_hit_coord_shift(give_site, direction, blast_hsp):
    """
    give_site = 100 (1-base)
    direction = 'query_to_subject'
    blast_hsp = BlastHsp by xuyuxing
    """

    q_seq = blast_hsp.Hsp_qseq
    s_seq = blast_hsp.Hsp_hseq
    midline = blast_hsp.Hsp_midline

    q_from = blast_hsp.Hsp_query_from
    q_to = blast_hsp.Hsp_query_to
    s_from = blast_hsp.Hsp_hit_from
    s_to = blast_hsp.Hsp_hit_to

    q_iterlap = no_gap_index(q_seq)
    s_iterlap = no_gap_index(s_seq)

    if direction == 'query_to_subject':
        if give_site < q_from:
            give_site = q_from
        if give_site > q_to:
            give_site = q_to

        give_site_index = give_site - q_from

        s_start, s_end, (a_start, a_end) = list(
            q_iterlap.find((give_site_index, give_site_index)))[0]
        aln_site = give_site_index - s_start + a_start

        output_site = len(
            s_seq[0:aln_site + 1].replace("-", "")) + s_from - 1
    elif direction == 'subject_to_query':
        if give_site < s_from:
            give_site = s_from
        if give_site > s_to:
            give_site = s_to

        give_site_index = give_site - s_from

        s_start, s_end, (a_start, a_end) = list(
            s_iterlap.find((give_site_index, give_site_index)))[0]
        aln_site = give_site_index - s_start + a_start

        output_site = len(
            q_seq[0:aln_site + 1].replace("-", "")) + q_from - 1

    return output_site


def hsp_judgment(blast_hsp):
    q_len = int(blast_hsp.query.qLen)
    s_len = int(blast_hsp.subject.Hit_len)

    if blast_hsp.Hsp_query_from == 1 and blast_hsp.Hsp_hit_from == 1 and q_len == blast_hsp.Hsp_query_to and s_len == blast_hsp.Hsp_hit_to and blast_hsp.Hsp_align_len == blast_hsp.Hsp_query_to and blast_hsp.Hsp_identity == blast_hsp.Hsp_query_to and blast_hsp.Hsp_gaps == 0:
        return True, 0, "perfect hit"

    elif blast_hsp.Hsp_identity / blast_hsp.Hsp_align_len < 0.95 or (
            blast_hsp.Hsp_query_to - blast_hsp.Hsp_query_from + 1) / q_len < 0.9:
        return False, 2, "low identity or coverage"

    else:
        return True, 1, "good hit"


def blat_hit_judgment(blat_hit, old_cdna_seq, new_genome_fasta, tmp_dir):
    # print(blat_hit)
    blat_cdna_seq = extract_give_feature_seq("match_part", blat_hit.sub_feature, blat_hit.chr_id,
                                             blat_hit.strand, new_genome_fasta)

    blast_results = blastn_pair_running(('query', old_cdna_seq), ('subject', blat_cdna_seq),
                                        tmp_dir, False)

    if len(blast_results['query'].hit) < 1:
        return False, None, "No blast hit", None

    good_hsp = []
    for hsp in blast_results['query'].hit[0].hsp:
        good_hsp_flag, note_code, note_hsp = hsp_judgment(hsp)
        if good_hsp_flag:
            good_hsp.append((hsp, note_code, note_hsp))

    if len(good_hsp) < 1:
        return False, None, "No good hit", None

    best_hsp = sorted(good_hsp, key=lambda x: x[1])[0]

    return True, best_hsp[0], best_hsp[2], blat_cdna_seq


def adjust_cds_phaes(cDNA_feature_list, phase):
    """
    cDNA_feature_list = [('five_prime_UTR',(1,20)),('CDS',(21,40)),('three_prime_UTR',(41,50))]
    cDNA_feature_list = [('five_prime_UTR',(1,10)),('CDS',(21,40)),('three_prime_UTR',(41,50))]
    phase = 1
    """

    for i in range(len(cDNA_feature_list)):
        type_tmp, range_tmp = cDNA_feature_list[i]
        if type_tmp == 'CDS':
            old_start = cDNA_feature_list[i][1][0]

            new_start = old_start + phase
            cDNA_feature_list[i] = (
                'CDS', (new_start, cDNA_feature_list[i][1][1]))

            if i > 0 and old_start - cDNA_feature_list[i - 1][1][1] == 1:
                new_end = new_start - 1
                cDNA_feature_list[i - 1] = (
                    cDNA_feature_list[i - 1][0], (cDNA_feature_list[i - 1][1][0], new_end))

            break

    return cDNA_feature_list


def generate_new_feature(mRNA, phase, blat_cdna_seq, best_hsp, blat_hit):
    # read old cDNA structure
    feature_list = cDNA_feature_coord(mRNA)

    # adjust phase
    feature_list = adjust_cds_phaes(feature_list, phase)

    # map old cDNA to new cDNA
    new_cDNA_feature_list = []
    for type_tmp, range_tmp in feature_list:
        start_tmp = blast_hit_coord_shift(
            range_tmp[0], 'query_to_subject', best_hsp)
        end_tmp = blast_hit_coord_shift(
            range_tmp[1], 'query_to_subject', best_hsp)

        new_cDNA_feature_list.append((type_tmp, (start_tmp, end_tmp)))

    # remove bad feature
    new_cDNA_feature_filterd_list = []
    for type_tmp, range_tmp in new_cDNA_feature_list:
        if range_tmp[1] - range_tmp[0] == 0:
            continue
        new_cDNA_feature_filterd_list.append((type_tmp, range_tmp))

    # test cds is good
    cds_seq = ""
    for type_tmp, range_tmp in new_cDNA_feature_filterd_list:
        if not type_tmp == 'CDS':
            continue

        cds_seq += blat_cdna_seq[range_tmp[0] - 1:range_tmp[1]]

    good_orf, phase, aa_seq = cds_judgment(cds_seq)

    if good_orf:

        new_cDNA_feature_filterd_phased_list = adjust_cds_phaes(
            new_cDNA_feature_filterd_list, phase)

        # get new genome feature
        genome_feature_list = []
        for type_tmp, range_tmp in new_cDNA_feature_filterd_phased_list:
            start_tmp = cDNA_coord_shift(
                range_tmp[0], 'cDNA_to_Genome', blat_hit.sub_feature)
            end_tmp = cDNA_coord_shift(
                range_tmp[1], 'cDNA_to_Genome', blat_hit.sub_feature)

            # print(range_tmp,(start_tmp, end_tmp))

            genome_feature_list.append(
                (type_tmp, (start_tmp, end_tmp)))

        # exon get
        exon_range = []
        for type_tmp, range_tmp in genome_feature_list:
            exon_range.append(range_tmp)

        exon_range = merge_intervals(exon_range, True)

        for i in exon_range:
            genome_feature_list.append(('exon', i))

        # change to GenomeFeature
        named_genome_feature_list = []
        for type_name, type_tag in [('CDS', 'cds'), ('exon', 'exon'), ('five_prime_UTR', 'utr5p'),
                                    ('three_prime_UTR', 'utr3p')]:
            type_list = sorted(
                [i for i in genome_feature_list if i[0] == type_name], key=lambda x: x[1][0])
            for i in range(len(type_list)):
                named_genome_feature_list.append(
                    ("%s.%s%d" % (blat_hit.name, type_tag, i + 1), type_list[i][0], type_list[i][1]))

        sub_feature = []
        site_list = []
        for sub_gf_id, type_tmp, range_tmp in named_genome_feature_list:
            site_list.extend(list(range_tmp))
            sub_feature.append(old_GenomeFeature(name=sub_gf_id, type=type_tmp, chr_id=blat_hit.chr_id,
                                             strand=blat_hit.strand, start=range_tmp[0], end=range_tmp[1]))

        # add cds phase
        cds_len_sum = 0
        if blat_hit.strand == '+' or blat_hit.strand == 1:
            for sub_gf in sorted(sub_feature, key=lambda x: x.start, reverse=False):
                if sub_gf.type == 'CDS':
                    if cds_len_sum % 3 == 0:
                        phase_tmp = 0
                    else:
                        phase_tmp = 3 - (cds_len_sum % 3)
                    sub_gf.phase = phase_tmp
                    cds_len_sum += abs(sub_gf.start - sub_gf.end) + 1
        elif blat_hit.strand == '-' or blat_hit.strand == -1:
            for sub_gf in sorted(sub_feature, key=lambda x: x.start, reverse=True):
                if sub_gf.type == 'CDS':
                    plus_phase = 3 - (cds_len_sum % 3)
                    if plus_phase == 0:
                        phase_tmp = 0
                    else:
                        phase_tmp = 3 - plus_phase
                    sub_gf.phase = phase_tmp
                    cds_len_sum += abs(sub_gf.start - sub_gf.end) + 1

        mRNA_start = min(site_list)
        mRNA_end = max(site_list)

        blat_mRNA = old_GenomeFeature(name=blat_hit.name, type="mRNA", chr_id=blat_hit.chr_id,
                                  strand=blat_hit.strand,
                                  start=mRNA_start, end=mRNA_end, sub_feature=sub_feature)

        blat_mRNA.blat_hit = blat_hit
        blat_mRNA.good_cds = True

    else:

        blat_mRNA = old_GenomeFeature(name=blat_hit.name, type="mRNA", chr_id=blat_hit.chr_id,
                                  strand=blat_hit.strand,
                                  start=0, end=0, sub_feature=None)
        blat_mRNA.good_cds = False
        blat_mRNA.blat_hit = blat_hit

    return blat_mRNA


def map_old_gene_to_new_assembly(mRNA, blat_hit_list, old_chr_id, old_genome_fasta_file, new_genome_fasta_file,
                                 tmp_dir):
    mRNA_tmp_dir = tmp_dir + "/" + mRNA.id
    mkdir(mRNA_tmp_dir, True)

    pyb_file = mRNA_tmp_dir + "/blat_mRNA.pyb"
    if not os.path.exists(pyb_file):

        # make sure old gene model is good, means cds is real ORF
        cds_seq = extract_give_feature_seq('CDS', mRNA.sub_features, old_chr_id, mRNA.strand,
                                           old_genome_fasta_file)

        good_orf, phase, aa_seq = cds_judgment(cds_seq)
        mRNA.good_cds = good_orf

        if good_orf:
            # get good blat hit
            old_cdna_seq = extract_give_feature_seq('exon', mRNA.sub_features, old_chr_id, mRNA.strand,
                                                    old_genome_fasta_file)
            good_blat = []
            for blat_hit in blat_hit_list:
                good_blat_flag, best_hsp, best_hsp_note, blat_cdna_seq = blat_hit_judgment(blat_hit,
                                                                                           old_cdna_seq,
                                                                                           new_genome_fasta_file,
                                                                                           mRNA_tmp_dir)

                blat_hit.hsp_note = best_hsp_note

                # print(best_hsp_note)

                if good_blat_flag:
                    good_blat.append(
                        (blat_hit, best_hsp, blat_cdna_seq))

            blat_mRNA_list = []
            for blat_hit, best_hsp, blat_cdna_seq in good_blat:
                blat_mRNA = generate_new_feature(
                    mRNA, phase, blat_cdna_seq, best_hsp, blat_hit)
                mRNA.chr_id = old_chr_id
                blat_mRNA.old_mRNA = mRNA
                blat_mRNA.best_hsp = best_hsp
                blat_mRNA_list.append(blat_mRNA)
        else:
            blat_mRNA_list = None

        OUT = open(pyb_file, 'wb')
        pickle.dump(blat_mRNA_list, OUT)
        OUT.close()

    else:
        TEMP = open(pyb_file, 'rb')
        blat_mRNA_list = pickle.load(TEMP)
        TEMP.close()

    return blat_mRNA_list


def cDNA2Genome_main(args):

    if args.tmp_dir is None:
        args.tmp_dir = os.getcwd() + '/tmp'

    mkdir(args.tmp_dir, True)
    print("tmp dir is : %s" % args.tmp_dir)

    pyb_file = args.tmp_dir + "/data_load.pyb"
    if not os.path.exists(pyb_file):
        # read old gff file

        old_gene_gff = {}
        with open(args.old_gff, 'r') as in_handle:
            for rec in GFF.parse(in_handle):
                for feature in rec.features:
                    feature.chr_id = rec.id
                    old_gene_gff[feature.id] = feature

        # read cds and cDNA blat results
        split_dir = args.tmp_dir + "/split"
        cdna_dict = split_and_load_blat_hit(args.cDNA_gff, 'cDNA', split_dir, threads=args.num_threads, target_per_file=1)
        # cdna_dict = blat_hit_load(args.cDNA_gff, 'cDNA')

        for old_mRNA_id in cdna_dict:
            for i in range(len(cdna_dict[old_mRNA_id])):
                cdna_dict[old_mRNA_id][i].rank = i

        # cds_dict = blat_hit_load(args.CDS_gff, 'CDS')

        OUT = open(pyb_file, 'wb')
        pickle.dump((old_gene_gff, cdna_dict), OUT)
        OUT.close()
    else:
        TEMP = open(pyb_file, 'rb')
        old_gene_gff, cdna_dict = pickle.load(TEMP)
        TEMP.close()

    # just use cDNA and old gff
    blat_mRNA_dir = args.tmp_dir + "/blat_mRNA"
    mkdir(blat_mRNA_dir, True)
    args_list = []

    for old_gene_id in old_gene_gff:
        # random.seed(1234)
        # for old_gene_id in random.sample(list(old_gene_gff.keys()), 200):
        #     print(old_gene_id)
        # old_gene_id = 'evm.TU.scaffold_43.136'

        gene = old_gene_gff[old_gene_id]

        # chr_id = gene.chr_id
        for mRNA in gene.sub_features:
            if not mRNA.id in cdna_dict:
                continue

            args_list.append(
                (mRNA, cdna_dict[mRNA.id], gene.chr_id, args.old_genome, args.new_genome, blat_mRNA_dir))

            """
            mRNA, blat_hit_list, old_chr_id, old_genome_fasta_file, new_genome_fasta_file, tmp_dir = mRNA, cdna_dict[mRNA.id], gene.chr_id, args.old_genome, args.new_genome, blat_mRNA_dir
            
            """

    map_output = multiprocess_running(
        map_old_gene_to_new_assembly, args_list, args.num_threads)

    huge_blat_mRNA_list = []
    for i in map_output:
        if map_output[i]['output'] is None:
            continue
        for j in map_output[i]['output']:
            if j.good_cds:
                huge_blat_mRNA_list.append(j)

    blat_mRNA_cluster = cluster_of_chr_loci(huge_blat_mRNA_list, 0.5)

    write_blat_mRNA = []
    for contig in blat_mRNA_cluster:
        for strand in blat_mRNA_cluster[contig]:
            group_dict = blat_mRNA_cluster[contig][strand]
            for group_id in group_dict:
                gf_list = group_dict[group_id]['list']
                best_gf = sorted(
                    gf_list, key=lambda x: x.best_hsp.Hsp_score, reverse=True)[0]
                best_gf.used = True
                write_blat_mRNA.append(best_gf)

    write_blat_gene = []
    for i in write_blat_mRNA:
        old_name = i.name
        gene_name = old_name + ".gene"
        mRNA_name = old_name + ".mRNA"

        gene = old_GenomeFeature(name=gene_name, type='gene',
                             chr_loci=i.chr_loci, sub_feature=[i])

        write_blat_gene.append(gene)

    write_gff_file(write_blat_gene, args.output_gff, source="blat", sort=True)
    # write_GenomeFeature_to_GFF(
    #     write_blat_gene, args.output_gff, source="blat")
    #
    # # write report
    # blat_mRNA_cluster_hash = {}
    # for contig in blat_mRNA_cluster:
    #     for strand in blat_mRNA_cluster[contig]:
    #         group_dict = blat_mRNA_cluster[contig][strand]
    #         for group_id in group_dict:
    #             group_range = group_dict[group_id]['range'][0]
    #             group_loci = ChrLoci(chr_id=contig, strand=strand, start=group_range[0], end=group_range[1])
    #             gf_list = group_dict[group_id]['list']
    #             for i in gf_list:
    #                 blat_mRNA_cluster_hash[i] = group_loci
    #
    # for i in map_output:
    #     mRNA, blat_hit_list, tmp1, tmp2, tmp3, tmp4 = map_output[i]['args']
    #     blat_mRNA_list = map_output[i]['output']
    #
    #     # old mRNA info
    #     if blat_mRNA_list is None:
    #         # old mRNA is not good cds
    #         old_cds_good = False
    #         blat_hit_passed_num = None
    #         blat_hit_total_num = len(cdna_dict[mRNA.id])
    #     else:
    #         old_cds_good = True
    #         blat_hit_passed_num = len(blat_mRNA_list)
    #         blat_hit_total_num = len(cdna_dict[mRNA.id])
    #
    #     # blat hit info
    #     if old_cds_good and blat_hit_passed_num > 0:
    #         mRNA = blat_mRNA_list[0].old_mRNA
    #         for blat_mRNA in blat_mRNA_list:
    #             blat_hit = blat_mRNA.blat_hit
    #             blat_hit_rank = blat_hit.rank
    #             blat_hit_note = blat_hit.hsp_note
    #             blat_cds_good = blat_mRNA.good_cds
    #             if hasattr(blat_mRNA, 'used'):
    #                 blat_used = blat_mRNA.used
    #             else:
    #                 blat_used = False
    #
    #             print(mRNA.id, old_cds_good, blat_hit_total_num, blat_hit_passed_num, blat_hit.name,blat_hit.get_fancy_name(),
    #                   blat_hit_rank, blat_hit_note, blat_cds_good, blat_used)


def Protein2Genome_main(args):

    # prepare args

    if not args.tmp_path:
        args.tmp_path = os.path.dirname(
            os.path.abspath(args.genome_file)) + "/tmp"

    mkdir(args.tmp_path, True)

    logger = logging_init("HomoAnno", args.log_file)

    args_info_string = "Argument list:\n"
    attrs = vars(args)

    for item in attrs.items():
        args_info_string = args_info_string + ("%s: %s\n" % item)

    logger.info(args_info_string)

    # get query sequence
    logger.info("Step1: split query sequence to many small files")

    query_gene_dict = read_fasta_by_faidx(args.protein_fasta)

    ready_seq_num = 0
    query_file_list = []
    for file_name in os.listdir(args.tmp_path):
        matchobj = re.match('^query_(.*)fa$', file_name)
        if matchobj:
            query_file_name = args.tmp_path + "/" + file_name
            ready_seq_num = ready_seq_num + \
                len(read_fasta_by_faidx(query_file_name))
            query_file_list.append(query_file_name)

    if not len(query_gene_dict) == ready_seq_num:
        logger.info("make new query fasta files")
        file_num = 0
        query_file_list = []
        query_file_name = "%s/query_%d.fa" % (args.tmp_path, file_num)
        query_file_list.append(query_file_name)
        f = open(query_file_name, 'w')

        seq_in_this_file_num = 0
        writed_seq_num = 0
        for gene_id in query_gene_dict:
            gene = query_gene_dict[gene_id]
            f.write(">%s\n%s\n" % (gene_id, gene.seq))
            writed_seq_num += 1
            seq_in_this_file_num += 1
            if seq_in_this_file_num == 2 and seq_in_this_file_num != len(query_gene_dict):
                f.close()
                file_num = file_num + 1
                query_file_name = "%s/query_%d.fa" % (
                    args.tmp_path, file_num)
                query_file_list.append(query_file_name)
                f = open(query_file_name, 'w')
                seq_in_this_file_num = 0
        f.close()
    else:
        logger.info("ready have query fasta files")

    # genblasta and genewise
    logger.info(
        "Step2: running genblasta and genewise to annotate genome again")

    # genblasta
    genblasta_finished_flag = True
    unfinished_query_list = []
    for q_file in query_file_list:
        if not os.path.exists(q_file + ".gba"):
            unfinished_query_list.append(q_file)
            genblasta_finished_flag = False

    if not genblasta_finished_flag:
        logger.info("genblasta: %d jobs in %d pool" %
                    (len(unfinished_query_list), args.num_threads))

        args_list = [(i, args.genome_file, i + ".gba", args.gene_coverage, args.genblasta_hit_num) for i in
                     unfinished_query_list]
        genblasta_out = multiprocess_running(
            genblasta_run, args_list, args.num_threads, log_file=args.log_file)

        logger.info("genblasta is finished")
    else:
        logger.info("genblasta is already finished")

    genewise_finished_flag = True
    unfinished_query_list = []
    for q_file in query_file_list:
        if not os.path.exists(q_file + ".wise.pep"):
            unfinished_query_list.append(q_file)
            genewise_finished_flag = False

    if not genewise_finished_flag:
        logger.info("genewise: %d jobs in %d pool" %
                    (len(unfinished_query_list), args.num_threads))
        mkdir(args.tmp_path + "/genewise_run/", False)
        args_list = [
            (
                i, args.genome_file, i + ".gba", args.tmp_path +
                "/genewise_run/", args.flank_range,
                i + ".wise",
                False)
            for
            i in
            unfinished_query_list]
        genewise_running_out = multiprocess_running(genblasta_to_genewise, args_list, args.num_threads,
                                                    log_file=args.log_file)

        genewise_error_query_file_list = []
        for i in genewise_running_out:
            if genewise_running_out[i]['output'] == "error":
                genewise_error_query_file_list.append(
                    genewise_running_out[i]['args'][0])

        logger.info("error file name:\n")

        for i in genewise_error_query_file_list:
            logger.info(i + "\n")

        logger.info("\n")

        logger.info("genewise is finished, %d seq get genewise error" %
                    len(genewise_error_query_file_list))

    else:
        logger.info("genewise is already finished")

    logger.info("genblasta and genewise OK")


if __name__ == '__main__':

    # Protein2Genome

    class abc():
        pass

    args = abc()

    args.genome_file = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/Gel.genome.v1.0.fasta"
    args.protein_fasta = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/GWHAAEX00000000.Protein.faa"
    args.output_prefix = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/HomoAnno"
    args.log_file = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/log"
    args.tmp_path = None
    args.num_threads = 10
    args.gene_coverage = 0.5
    args.genblasta_hit_num = 20
    args.flank_range = 1000

    Protein2Genome_main(args)

    # cDNA2Genome
    """
    blat Gel.genome.v1.0.fasta /dev/null /dev/null -tileSize=11 -makeOoc=11.ooc -repMatch=1024

    blat Gel.genome.v1.0.fasta Gel.yuan.cDNA.fa -q=rna -dots=100  -maxIntron=500000 -out=psl -ooc=11.ooc Gel.yuan.cDNA.psl
    blat Gel.genome.v1.0.fasta Gel.yuan.CDS.fa -q=rna -dots=100  -maxIntron=500000 -out=psl -ooc=11.ooc Gel.yuan.CDS.psl

    blat2gff.pl < Gel.yuan.cDNA.psl > Gel.yuan.cDNA.gff
    blat2gff.pl < Gel.yuan.CDS.psl > Gel.yuan.CDS.gff

    """

    class abc():
        pass

    args = abc()
    args.cDNA_gff = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/Gel.yuan.cDNA.gff"
    args.cDNA_fasta = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/Gel.yuan.cDNA.fa"
    args.old_gff = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/GWHAAEX00000000.rename.gff"
    args.old_genome = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/GWHAAEX00000000.genome.fasta"
    args.new_genome = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/Gel.genome.v1.0.fasta"
    args.tmp_dir = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/tmp"
    args.num_threads = 10
    args.output_gff = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/blat.output.gff"

    cDNA2Genome_main(args)

    ###
    class abc():
        pass

    args = abc()
    args.cDNA_gff = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca2/purge/MapOldAnno/head.cDNA.c0.9.gff"
    args.old_gff = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca/T99112N0.genome.gff3"
    args.old_genome = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca/T99112N0.genome.fasta"
    args.new_genome = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca2/purge/purge_haplotigs/curated.fasta"
    args.tmp_dir = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca2/purge/MapOldAnno/tmp"
    args.num_threads = 80
    args.output_gff = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca2/purge/MapOldAnno/MapOldAnno.gff"

    cDNA2Genome_main(args)


import re
import os
import time
import sys

__author__ = 'Yuxing Xu'

if __name__ == '__main__':
    import argparse

    # argument parse
    # ---------------------------------------------------------
    parser = argparse.ArgumentParser(
        prog='SeqParser',
    )

    subparsers = parser.add_subparsers(
        title='subcommands', dest="subcommand_name")

    # argparse for FastaFormatClean
    parser_a = subparsers.add_parser('FastaFormatClean',
                                     help='clean fasta format to a good way\n',
                                     description='')

    parser_a.add_argument('raw_fasta_file', type=str, help='raw fasta file')
    parser_a.add_argument('output_fasta_file', type=str,
                          help='output fasta file')
    parser_a.add_argument("-t", "--seq_type", help="what type sequences you have? (default:prot)", default="prot",
                          choices=['prot', 'nucl'])
    parser_a.add_argument("-d", "--degenerate_sites_allowed",
                          help="allow degenerate sites", action='store_true')
    parser_a.add_argument("-g", "--gap_allowed",
                          help="allow gap sites", action='store_true')
    parser_a.add_argument("-s", "--translation_stop_allowed",
                          help="allow translation stop sites", action='store_true')
    parser_a.add_argument("-w", "--wrap", help="all lines of text be shorter than warp characters in length", type=int,
                          default=75)
    parser_a.add_argument("-r", "--replace_flag",
                          help="if find a unknown or not allowed site, should we replace to any site? (default: no and skip sequence)",
                          action='store_true')
    parser_a.add_argument("-n", "--full_name_flag",
                          help="use full sequence name in output", action='store_true')
    parser_a.add_argument('-l', "--log_file", type=str,
                          help='path for log file (default:None)', default=None)

    # argparse for FastaStats
    parser_a = subparsers.add_parser('FastaStats',
                                     help='Stats fasta file like N50 and so on\n',
                                     description='')

    parser_a.add_argument('fasta_file', type=str, help='fasta file')
    parser_a.add_argument('-o', "--output_file", type=str, help='output file')
    parser_a.add_argument(
        "-old", "--old_way", help="use old way from xuyuxing", action='store_true')

    # argparse for FastaToSQL
    parser_a = subparsers.add_parser('FastaToSQL',
                                     help='store a fasta file into a sqlite database\n',
                                     description='store a fasta file into a sqlite database')

    parser_a.add_argument('fasta_file', type=str, help='a fasta file')
    parser_a.add_argument('sqlite_file', type=str,
                          help='output sqlite database')
    parser_a.add_argument('-l', "--log_file", type=str,
                          help='path for log file (default:None)', default=None)
    parser_a.add_argument("-gzip", "--gzip_flag",
                          help="if fasta is gzip file", action='store_true')

    # argparse for FastaByID
    parser_a = subparsers.add_parser('FastaByID',
                                     help='Extract fasta records by IDs from a database fasta file\n',
                                     description='Extract fasta record by ID from a database fasta file')

    parser_a.add_argument('db_fasta_file', type=str,
                          help='a database fasta file')
    parser_a.add_argument('-i', "--ID_list", type=str,
                          help='ID list like ID1,ID2,ID3')
    parser_a.add_argument('-f', '--ID_file', type=str,
                          help='file with ID in a column')
    parser_a.add_argument('-o', "--output_file", type=str, help='output file')
    parser_a.add_argument('-l', "--log_file", type=str,
                          help='path for log file (default:None)', default=None)
    parser_a.add_argument(
        "-old", "--old_way", help="use old way from xuyuxing", action='store_true')
    parser_a.add_argument("-v", "--invert_flag",
                          help="use old way from xuyuxing", action='store_true')

    # argparse for FastaByGroup
    parser_a = subparsers.add_parser('FastaByGroup',
                                     help='Extract many subset fasta records by groups from a database fasta file\n',
                                     description='')

    parser_a.add_argument("-n", "--name_flag",
                          help="if group file frist column is not name of group, auto make group name",
                          action='store_false')
    parser_a.add_argument('group_file', type=str,
                          help='a csv file which different rows represent different groups and the first column should be the group name')
    parser_a.add_argument('db_fasta_file', type=str,
                          help='a database fasta file')
    parser_a.add_argument('out_dir', type=str, help='output dir')

    # argparse for FastaRename
    parser_a = subparsers.add_parser('FastaRename',
                                     help='Extract fasta records by IDs and rename them\n',
                                     description='Extract fasta records by IDs and rename them')

    parser_a.add_argument('db_fasta_file', type=str,
                          help='a database fasta file')
    parser_a.add_argument('-f', '--ID_map_file', type=str,
                          help='file with ID in first column and new ID in a second column')
    parser_a.add_argument('-o', "--output_file", type=str, help='output file')
    parser_a.add_argument('-p', "--seq_prefix", type=str,
                          help='rename seq by a given prefix')
    parser_a.add_argument(
        "-old", "--old_way", help="use old way from xuyuxing", action='store_true')

    # argparse for SeqMerge
    parser_a = subparsers.add_parser('SeqMerge',
                                     help='merge give sequences into a single sequence\n',
                                     description='merge give sequences into a single sequence')

    parser_a.add_argument('ID_merge_file', type=str,
                          help='file with ID in first column and merged new ID in a second column')
    parser_a.add_argument('db_fasta_file', type=str,
                          help='a database fasta file')
    parser_a.add_argument('output_file', type=str, help='output file')

    # argparse for GeneStructure
    parser_a = subparsers.add_parser('GeneStructure',
                                     help='Extract gene sequence by they structure\n')

    parser_a.add_argument('genome_fasta_file', type=str,
                          help='reference genome')
    parser_a.add_argument('gff_file', type=str,
                          help='reference genome gff file')
    parser_a.add_argument('gene_id', type=str, help='gene id which you want')
    parser_a.add_argument('-o', "--output_file", type=str, help='output file')

    # argparse for SeqLength
    parser_a = subparsers.add_parser('SeqLength',
                                     help='Get the length of each sequence in fasta file\n',
                                     description='Get the length of each sequence in fasta file')

    parser_a.add_argument('seq_file', type=str,
                          help='a fasta or fastq file as input')
    parser_a.add_argument(
        '-s', "--stat", help='make a histogram for seq length', action='store_true')
    parser_a.add_argument('-fq', "--fq_flag",
                          help='input is fastq file', action='store_true')
    parser_a.add_argument('-p', "--picture", type=str,
                          help='make a histogram picture for seq length picture file path need here', default="")
    parser_a.add_argument('-o', "--output_file", type=str, help='output file')

    # argparse for FilterByLen
    parser_a = subparsers.add_parser('FilterByLen',
                                     help='Filter a fasta file by length of sequence\n',
                                     description='Filter a fasta file by length of sequence')

    parser_a.add_argument('seq_file', type=str, help='a fasta as input')
    parser_a.add_argument('output_file', type=str, help='output file')
    parser_a.add_argument('condition_string', type=str,
                          help='condition string, such as: "seq>100", "seq<100", "seq>=100", "seq<=100", "seq<100 and seq>50", "seq<100 or seq>200"')

    # argparse for SubFasta
    parser_a = subparsers.add_parser('SubFasta',
                                     help='Extract local sequence by ID, start, end, and strand site from a database fasta file\n',
                                     description='Extract local sequence by ID, start, end, and strand site from a database fasta file')

    parser_a.add_argument('db_fasta_file', type=str,
                          help='a database fasta file')
    parser_a.add_argument('-s', "--search_string", type=str,
                          help='search_string like Chr1:1-100:+')
    parser_a.add_argument('-f', "--input_file", type=str,
                          help='csv file [subseqname,refseqname,start,end,strand]')
    parser_a.add_argument('-b', "--input_a_bed_file", action='store_true')
    parser_a.add_argument(
        "-r", "--RNA", help="change T to U", action='store_true')
    parser_a.add_argument('-o', "--output_file", type=str, help='output file')

    # argparse for GCproportion
    parser_a = subparsers.add_parser('GCproportion',
                                     help='get GC proportion for a fasta file\n')

    parser_a.add_argument('fasta_file', type=str, help='a database fasta file')
    parser_a.add_argument('output_file', type=str, help='output file')

    # argparse for ReComp
    parser_a = subparsers.add_parser('ReComp',
                                     help='reverse and complement a sequence\n')

    parser_a.add_argument('input_fasta_file', type=str, help='path for input')
    parser_a.add_argument('output_fasta_file', type=str,
                          help='path for output')
    parser_a.add_argument('-r', "--only_reverse", action='store_true')
    parser_a.add_argument('-c', "--only_complement", action='store_true')

    # argparse for SeqEntropy
    parser_a = subparsers.add_parser('SeqEntropy',
                                     help='get seq entropy value\n')

    parser_a.add_argument('input_fasta', type=str, help='path for input')
    parser_a.add_argument('output_file', type=str, help='path for output')
    parser_a.add_argument('-w', "--word_size", type=int, default=3)

    # argparse for SplitFasta
    parser_a = subparsers.add_parser('SplitFasta',
                                     help='Split a big fasta file into some smaller file\n',
                                     description='Split a big fasta file into some smaller file')

    parser_a.add_argument('fasta_file', type=str,
                          help='a big fasta file as input')
    parser_a.add_argument('-r', "--records", type=int, default=None,
                          help='put -r sequences per output fasta file')
    parser_a.add_argument('-c', "--contig_model",
                          help="use contig model, store one contig in one file and named with contig",
                          action='store_true')
    parser_a.add_argument('-s', "--size", type=int,
                          default=None, help='put size per output fasta file')
    parser_a.add_argument('-o', "--output_prefix", type=str, default="new_",
                          help='prefix for output, default is "new_"')

    # argparse for FragmentGenome
    parser_a = subparsers.add_parser('FragmentGenome',
                                     help='cut genome to fragment')
    parser_a.add_argument(
        "genome_file", help="Path of genome file with fasta format", type=str)
    parser_a.add_argument(
        "output_file", help="Path of cutted genome file", type=str)
    parser_a.add_argument("step", help="step of genome cutting", type=int)
    parser_a.add_argument(
        "length", help="length of sequence of cutted genome", type=int)
    parser_a.add_argument("-f", "--shift_start", help="fragment start at which site, (default as 0, meaning not shift)",
                          default=0, type=int)
    parser_a.add_argument("-s", "--Consider_scaffold", help="If consider scaffold in the genome (default as True)",
                          default=True)
    parser_a.add_argument("-e", "--entropy_threshold", help="filter simple repeat by seq entropy (default as 3)",
                          default=3, type=int)

    # argparse for ReGetContig
    parser_a = subparsers.add_parser('ReGetContig',
                                     help='cut scaffold to contig')
    parser_a.add_argument(
        "scaff_file", help="Path of scaffold version to contig again", type=str)
    parser_a.add_argument(
        "output_gff3", help="Path of cutted contig gff file", type=str)
    parser_a.add_argument(
        "output_fasta", help="Path of cutted contig fasta file", type=str)
    parser_a.add_argument(
        "-t", "--threads", help="threads number (default as 5)", default=5, type=int)

    # argparse for AlnStat
    parser_a = subparsers.add_parser('AlnStat', help='Alignment statistics tool\n',
                                     description='Alignment statistics tool')

    parser_a.add_argument('aln_file', type=str,
                          help='Alignments in fasta file')
    parser_a.add_argument('output_file', type=str, help='output file')
    parser_a.add_argument('-q', "--query", type=str,
                          help='query seq for marker record, default is marking in all colunm')

    # argparse for FastqSize
    parser_a = subparsers.add_parser('FastqSize',
                                     help='count base and reads number in a fastq file')

    parser_a.add_argument('-1', "--fastq_1", type=str, help='left end fastq')
    parser_a.add_argument('-2', "--fastq_2", type=str,
                          help='right end fastq (option)')
    parser_a.add_argument("-gzip", "--gzip_flag",
                          help="if fastq is gzip file", action='store_true')

    # argparse for PreFastqByID
    parser_a = subparsers.add_parser('PreFastqByID',
                                     help='build a sqlite3 database for fastq records\n',
                                     description='build a sqlite3 database for fastq records')

    parser_a.add_argument('-1', "--fastq_1", type=str, help='left end fastq')
    parser_a.add_argument('-2', "--fastq_2", type=str,
                          help='right end fastq (option)')
    parser_a.add_argument('-o', "--output_file", type=str,
                          help='name of output file')
    parser_a.add_argument("-gzip", "--gzip_flag",
                          help="if fastq is gzip file", action='store_true')

    # argparse for FastqByID
    parser_a = subparsers.add_parser('FastqByID',
                                     help='Extract fastq records by IDs from a database fastq file\n',
                                     description='Extract fastq record by ID from a database fastq file')

    parser_a.add_argument('db_file', type=str,
                          help='database file made by PreFastqByID')
    parser_a.add_argument('-i', "--ID_list", type=str,
                          help='ID list like ID1,ID2,ID3')
    parser_a.add_argument('-f', '--ID_file', type=str,
                          help='file with ID in a column')
    parser_a.add_argument('-o', "--output_prefix",
                          type=str, help='prefix of output file')

    # argparse for PreExtractNr
    parser_a = subparsers.add_parser('PreExtractNr',
                                     help='convert nr fasta file to a sqlite3 database for extract record\n',
                                     description='convert nr fasta file to a sqlite3 database for extract record')

    parser_a.add_argument('nr_fasta_file', type=str,
                          help='NCBI nr database in the fasta format')
    parser_a.add_argument('sql3_db_file', type=str,
                          help='sqlite3 which output by there command')

    # argparse for ExtractNr
    parser_a = subparsers.add_parser('ExtractNr',
                                     help='Extract local sequence by ID, start, end, and strand site from a database fasta file\n',
                                     description='Extract local sequence by ID, start, end, and strand site from a database fasta file')
    parser_a.add_argument('db_fasta_file', type=str,
                          help='a database fasta file')
    parser_a.add_argument('-i', "--ID_list", type=str,
                          help='ID list like ID1,ID2,ID3')
    parser_a.add_argument('-f', '--ID_file', type=str,
                          help='file with ID in a column')
    parser_a.add_argument("-l", "--long_name",
                          help="use full sequence name", action='store_false')
    parser_a.add_argument('-o', "--output_file", type=str, help='output file')

    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    # ccommand detail

    # FastaByID
    if args_dict["subcommand_name"] == "FastaByID":
        from src.xuyuxing.tools.seqtools import FastaByID_main
        FastaByID_main(args)

    # FastaByGroup
    elif args_dict["subcommand_name"] == "FastaByGroup":
        from src.xuyuxing.tools.seqtools import FastaByGroup_main
        FastaByGroup_main(args)

    # subfasta
    elif args_dict["subcommand_name"] == "SubFasta":
        from src.xuyuxing.tools.seqtools import subfasta_main
        subfasta_main(args)

    # PreExtractNr
    elif args_dict["subcommand_name"] == "PreExtractNr":
        from src.xuyuxing.tools.seqtools import PreExtractNr_main
        PreExtractNr_main(args)

    # ExtractNr
    elif args_dict["subcommand_name"] == "ExtractNr":
        from src.xuyuxing.tools.seqtools import ExtractNr_main
        ExtractNr_main(args)

    # SeqLength
    elif args_dict["subcommand_name"] == "SeqLength":
        from src.xuyuxing.tools.seqtools import SeqLength_main
        SeqLength_main(args)

    # SplitFasta
    elif args_dict["subcommand_name"] == "SplitFasta":
        from src.xuyuxing.tools.seqtools import SplitFasta_main
        SplitFasta_main(args)

    # AlnStat
    elif args_dict["subcommand_name"] == "AlnStat":
        from src.xuyuxing.tools.seqtools import AlnStat_main
        AlnStat_main(args)

    # FastqByID
    elif args_dict["subcommand_name"] == "FastqByID":
        from src.xuyuxing.tools.seqtools import FastqByID_main
        FastqByID_main(args)

    # PreFastqByID
    elif args_dict["subcommand_name"] == "PreFastqByID":
        from src.xuyuxing.tools.seqtools import PreFastqByID_main
        PreFastqByID_main(args)

    # FastaRename
    elif args_dict["subcommand_name"] == "FastaRename":
        from src.xuyuxing.tools.seqtools import FastaRename_main
        FastaRename_main(args)

    # FilterByLen
    elif args_dict["subcommand_name"] == "FilterByLen":
        from src.xuyuxing.tools.seqtools import FilterByLen_main
        FilterByLen_main(args)

    # FastaToSQL
    elif args_dict["subcommand_name"] == "FastaToSQL":
        from src.xuyuxing.tools.seqtools import FastaToSQL_main
        FastaToSQL_main(args)

    # FastaFormatClean
    elif args_dict["subcommand_name"] == "FastaFormatClean":
        from src.xuyuxing.tools.seqtools import FastaFormatClean_main
        FastaFormatClean_main(args)

    # ReComp
    elif args_dict["subcommand_name"] == "ReComp":
        from src.xuyuxing.tools.seqtools import ReComp_main
        ReComp_main(args)

    # FastqSize
    elif args_dict["subcommand_name"] == "FastqSize":
        from src.xuyuxing.tools.seqtools import FastqSize_main
        FastqSize_main(args)

    elif args_dict["subcommand_name"] == "FastaStats":
        from src.xuyuxing.tools.seqtools import FastaStats_main
        FastaStats_main(args)

    # ReGetContig
    elif args_dict["subcommand_name"] == "ReGetContig":
        from src.xuyuxing.tools.seqtools import ReGetContig_main
        ReGetContig_main(args)

    # GeneStructure
    elif args_dict["subcommand_name"] == "GeneStructure":
        from src.xuyuxing.tools.seqtools import GeneStructure_main
        GeneStructure_main(args)

    # SeqMerge
    elif args_dict["subcommand_name"] == "SeqMerge":
        from src.xuyuxing.tools.seqtools import SeqMerge_main
        SeqMerge_main(args)

    # GCproportion
    elif args_dict["subcommand_name"] == "GCproportion":
        from src.xuyuxing.tools.seqtools import GCproportion_main
        GCproportion_main(args)

    # SeqEntropy
    elif args_dict["subcommand_name"] == "SeqEntropy":
        from src.xuyuxing.tools.seqtools import SeqEntropy_main
        SeqEntropy_main(args)

    # FragmentGenome
    elif args_dict["subcommand_name"] == "FragmentGenome":
        from src.xuyuxing.tools.seqtools import FragmentGenome_main
        FragmentGenome_main(args)



if __name__ == '__main__':

    import argparse

    ###### argument parse
    parser = argparse.ArgumentParser(
        prog='Submitter',
    )

    subparsers = parser.add_subparsers(title='subcommands', dest="subcommand_name")

    # argparse for LocalMultiWork
    parser_a = subparsers.add_parser('LocalMultiWork',
                                     help='running many jobs by one machine')

    parser_a.add_argument('command_file', type=str, help='Path of command file for many jobs, each line is one command')
    parser_a.add_argument("-t", "--num_threads", help="num of threads (default:1)", default=1, type=int)
    parser_a.add_argument("-cwd", "--cwd_file",
                          help="Path of work dir file, each line in this file corresponds the cwd to each command in the command file.",
                          default=None, type=str)
    parser_a.add_argument("-l", "--log_file", help="path of log file", default=None, type=str)
    parser_a.add_argument("-s", "--silence", help="don't get error and stdout", action='store_true', default=False)
    parser_a.add_argument("-c", "--conda_env_mode", help="put cmd into a single shell file and with a given conda env",
                          default=None, type=str)

    # argparse for SGEMultiWork
    parser_a = subparsers.add_parser('SGEMultiWork',
                                     help='running many jobs by sge')

    parser_a.add_argument('command_file', type=str, help='Path of command file for many jobs, each line is one command')
    parser_a.add_argument("-wd", "--submit_dir", help="Path of submiter dir.", default=None, type=str)
    parser_a.add_argument("-cwd", "--cwd_file",
                          help="Path of work dir file, each line in this file corresponds the cwd to each command in the command file.",
                          default=None, type=str)
    parser_a.add_argument("-bq", "--base_qsub_args", help="base args for qsub", default="-V -b y -cwd -q all.q@@xyx",
                          type=str)
    parser_a.add_argument("-aq", "--add_qsub_args", help="add args for qsub", default="", type=str)
    parser_a.add_argument("-c", "--conda_env_mode", help="put cmd into a single shell file and with a given conda env",
                          default=None, type=str)

    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    #### command detail

    if args_dict["subcommand_name"] == "LocalMultiWork":
        import subprocess
        from multiprocessing import Pool
        import logging
        import time
        import os
        from lib.common.os import mkdir, cmd_run, multiprocess_running


        def log_print(print_string):
            time_tmp = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())
            print("%s\t\t\t%s\n" % (time_tmp, print_string))


        def logging_init(program_name, log_file=None):
            # create logger with 'program_name'
            logger = logging.getLogger(program_name)
            logger.setLevel(logging.DEBUG)
            # create formatter and add it to the handlers
            formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
            if not log_file is None:
                # create file handler which logs even debug messages
                fh = logging.FileHandler(log_file)
                fh.setLevel(logging.DEBUG)
                fh.setFormatter(formatter)
                logger.addHandler(fh)
            # create console handler with a higher log level
            ch = logging.StreamHandler()
            ch.setLevel(logging.ERROR)
            ch.setFormatter(formatter)
            # add the handlers to the logger
            logger.addHandler(ch)

            return logger


        def chunks(l, n):
            """Yield successive n-sized chunks from l."""
            for i in range(0, len(l), n):
                yield l[i:i + n]


        def cmd_run_rltime(cmd_string, stdout_file, stderr_file, cwd=None, silence=False):
            if silence:
                returncode = subprocess.call(cmd_string, cwd=cwd, shell=True)
            else:
                with open(stdout_file, 'w') as stdout_f:
                    with open(stderr_file, 'w') as stderr_f:
                        returncode = subprocess.call(cmd_string, cwd=cwd, shell=True, stdout=stdout_f, stderr=stderr_f)
            return returncode


        # def multiprocess_running(f, args_list, pool_num, slience=True):
        #     """
        #
        #     :param f: function name
        #     :param args_list: a list include every args in a tuple
        #     :param pool_num: process num of running at sametime
        #     :param slience:
        #     :return:
        #     """
        #
        #     if pool_num == 0:
        #         p = Pool()
        #     else:
        #         p = Pool(pool_num)
        #     p_dict = {}
        #     num = 0
        #
        #     for i in args_list:
        #         p_dict['ID_' + str(num)] = {
        #             'args': i,
        #             'output': None,
        #             'success': None,
        #             'p_object': p.apply_async(f, args=i)
        #         }
        #         num = num + 1
        #
        #     if not slience:
        #         print('Waiting for all subprocesses done...')
        #     p.close()
        #     p.join()
        #     if not slience:
        #         print('All subprocesses done.')
        #
        #     for i in p_dict:
        #         p_dict[i]['output'] = p_dict[i]['p_object'].get()
        #         p_dict[i]['success'] = p_dict[i]['p_object']._success
        #         del p_dict[i]['p_object']
        #
        #     return p_dict

        """
        class abc():
            pass
        
        args=abc()
        args.command_file = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/pasa/evm/commands.list"
        args.num_threads = 56
        args.cwd_file = None
        args.log_file = None
        args.conda_env_mode = "maker_p"
        """

        logger = logging_init("LocalMultiWork", args.log_file)

        logger.info("Parsing cmd and cwd file")
        command_dict = {}
        num = 0
        with open(args.command_file, 'r') as f:
            for each_line in f.readlines():
                ID_tmp = "ID_" + str(num)
                each_line = each_line.strip()
                command_dict[ID_tmp] = {"cmd": each_line}
                num = num + 1

        logger.info("Parsed cmd file")
        if not args.cwd_file is None:
            logger.info("Found cwd file")
            num = 0
            with open(args.cwd_file, 'r') as f:
                for each_line in f.readlines():
                    ID_tmp = "ID_" + str(num)
                    each_line = each_line.strip()
                    command_dict[ID_tmp]["cwd"] = each_line
                    num = num + 1
            logger.info("Parsed cwd file")
        else:
            logger.info("Not Found cwd file, cmd will running in current word dir")
            for ID_tmp in command_dict:
                command_dict[ID_tmp]["cwd"] = os.getcwd()

        if args.conda_env_mode:
            logger.info("Parsed conda env mode")
            tmp_dir = os.getcwd() + "/tmp"
            mkdir(tmp_dir)
            for cmd_id in command_dict:
                tmp_file_name = tmp_dir + "/" + cmd_id + ".sh"
                with open(tmp_file_name, 'w') as f:
                    f.write("source activate %s\n" % args.conda_env_mode)
                    f.write("cd %s\n" % command_dict[cmd_id]['cwd'])
                    f.write("%s\n" % command_dict[cmd_id]['cmd'])
                    f.write("conda deactivate\n")
                command_dict[cmd_id]['cmd'] = "bash %s" % tmp_file_name

        args_list = []
        for ID_tmp in command_dict:
            cmd = command_dict[ID_tmp]['cmd']
            cwd = command_dict[ID_tmp]['cwd']
            stdout = cwd + "/" + ID_tmp + ".out"
            stderr = cwd + "/" + ID_tmp + ".err"
            args_tuple = (cmd, stdout, stderr, cwd, args.silence)
            args_list.append(args_tuple)

        cmd_result = multiprocess_running(cmd_run_rltime, args_list, args.num_threads, log_file=args.log_file)

        # logger.info("#############cmd running#############")
        # round_num = 0
        # for round_tmp in chunks(list(command_dict.keys()), args.num_threads * 1):
        #     logger.info("Round %d:" % round_num)
        #     logger.info("Included %s to %s" % (round_tmp[0], round_tmp[-1]))
        #
        #     args_list = []
        #     for ID_tmp in round_tmp:
        #         cmd = command_dict[ID_tmp]['cmd']
        #         cwd = command_dict[ID_tmp]['cwd']
        #         stdout = cwd + "/" + ID_tmp + ".out"
        #         stderr = cwd + "/" + ID_tmp + ".err"
        #         args_tuple = (cmd, stdout, stderr, cwd)
        #         args_list.append(args_tuple)
        #
        #     cmd_result = multiprocess_running(cmd_run_rltime, args_list, args.num_threads)
        #
        #     tmp_num = 0
        #     for ID_tmp in cmd_result:
        #         raw_ID_tmp = round_tmp[tmp_num]
        #         sucess_flag = cmd_result[ID_tmp]['success']
        #         if sucess_flag:
        #             # print("%s is OK" % raw_ID_tmp)
        #             logger.info("%s is OK" % raw_ID_tmp)
        #         else:
        #             logger.info("%s is BAD" % raw_ID_tmp)
        #         tmp_num = tmp_num + 1
        #
        #     round_num = round_num + 1
        #
        # logger.info("#############All Finished#############")

    elif args_dict["subcommand_name"] == "SGEMultiWork":
        """
        class abc():
            pass

        args=abc()
        args.command_file = "/lustre/home/xuyuxing/Database/Phelipanche/annotation/evm/est/cufflinks/cmd.list"
        args.submit_dir = "/lustre/home/xuyuxing/Database/Phelipanche/annotation/evm/est/cufflinks"
        args.base_qsub_args = "-V -b y -cwd -q all.q@@xyx"
        args.add_qsub_args = ""
        args.conda_env_mode = "map_tools"
        args.cwd_file = None
        """
        import os
        import re
        from lib.xuyuxing.base.base_function import mkdir, cmd_run

        if args.submit_dir is None:
            args.submit_dir = os.getcwd()
        else:
            args.submit_dir = os.path.abspath(args.submit_dir)
        tmp_dir = args.submit_dir + "/job_submit_tmp"
        mkdir(tmp_dir, False)

        command_dict = {}
        num = 0
        with open(args.command_file, 'r') as f:
            for each_line in f.readlines():
                ID_tmp = "ID_" + str(num)
                each_line = each_line.strip()
                command_dict[ID_tmp] = {"cmd": each_line}
                num = num + 1

        if not args.cwd_file is None:
            num = 0
            with open(args.cwd_file, 'r') as f:
                for each_line in f.readlines():
                    ID_tmp = "ID_" + str(num)
                    each_line = each_line.strip()
                    command_dict[ID_tmp]["cwd"] = each_line
                    num = num + 1
        else:
            for ID_tmp in command_dict:
                command_dict[ID_tmp]["cwd"] = args.submit_dir

        sge_jobs_dict = {}
        with open(args.command_file, 'r') as f:
            for cmd_id in command_dict:
                tmp_file_name = tmp_dir + "/" + cmd_id + ".sh"
                with open(tmp_file_name, 'w') as o:
                    o.write("source activate %s\n" % args.conda_env_mode)
                    o.write("cd %s\n" % command_dict[cmd_id]['cwd'])
                    o.write("%s\n" % command_dict[cmd_id]['cmd'])
                    o.write("conda deactivate\n")

                cmd_string = "qsub %s %s bash %s" % (args.base_qsub_args, args.add_qsub_args, tmp_file_name)

                returncode, output, error = cmd_run(cmd_string, cwd=args.submit_dir, retry_max=1, silence=True)
                # print(output)

                job_id, job_name = re.findall(r'Your job (\d+) \("(\S+)"\) has been submitted', output)[0]
                sge_jobs_dict[job_id] = (job_id, job_name)

        all_job_done = False

        while not all_job_done:
            all_job_done = True
            for job_id in sge_jobs_dict:
                cmd_string = 'qstat -j %s' % job_id
                returncode, output, error = cmd_run(cmd_string, cwd=args.submit_dir, retry_max=1, silence=True,
                                                    log_file=None)
                if not error == 'Following jobs do not exist: \n%s\n' % job_id:
                    all_job_done = False

        print("all %d jobs finished!" % len(sge_jobs_dict))



from Bio.Phylo.BaseTree import Tree, Clade
from lib.common.fileIO import tsv_file_dict_parse
import os
import pickle
import lib.common.sqlite_command as sc
from lib.common.util import logging_init
import time
import sqlite3
import re

try:
    script_dir_path = os.path.split(os.path.realpath(__file__))[0]
    dfam_TE_classes_file = script_dir_path + "/../config_file/TEClasses.txt"
except:
    dfam_TE_classes_file = "/lustre/home/xuyuxing/python_project/Genome_work_tools/lib/config_file/TEClasses.txt"


class TE_Class(Clade):
    """
    dfam TE class from https://dfam.org/classification/tree
    """

    def __init__(self,
                 full_name=None,
                 title=None,
                 description=None,
                 hyperlink=None,
                 repeatmasker_equiv=None,
                 repbase_equiv=None,
                 wicker_equiv=None,
                 curcio_derbyshire_equiv=None,
                 piegu_equiv=None,
                 ):

        name = full_name.split(";")[-1]

        super(TE_Class, self).__init__(
            branch_length=None,
            name=name,
            clades=None,
            confidence=None,
            color=None,
            width=None,
        )

        self.info = {
            "full_name": full_name,
            "title": title,
            "description": description,
            "hyperlink": hyperlink,
        }

        self.lineage = tuple(full_name.split(";"))

        self.db_id_map = {
            "repeatmasker": repeatmasker_equiv,
            "repbase": repbase_equiv,
            "wicker": wicker_equiv,
            "curcio_derbyshire": curcio_derbyshire_equiv,
            "piegu": piegu_equiv,
        }

        if len(self.lineage) > 1:
            self.parent = self.lineage[-2]
        else:
            self.parent = None


def build_TE_class_tree(new_dfam_TE_classes_file=None):
    if new_dfam_TE_classes_file:
        file_info_dict = tsv_file_dict_parse(new_dfam_TE_classes_file)
    else:
        file_info_dict = tsv_file_dict_parse(dfam_TE_classes_file)

    clade_dict = {}
    for i in file_info_dict:
        repeatmasker_equiv = (
            file_info_dict[i]['repeatmasker_type'], file_info_dict[i]['repeatmasker_subtype'])

        tc = TE_Class(
            full_name=file_info_dict[i]['full_name'],
            title=file_info_dict[i]['title'],
            description=file_info_dict[i]['description'],
            hyperlink=file_info_dict[i]['hyperlink'],
            repeatmasker_equiv=repeatmasker_equiv,
            repbase_equiv=file_info_dict[i]['repbase_equiv'],
            wicker_equiv=file_info_dict[i]['wicker_equiv'],
            curcio_derbyshire_equiv=file_info_dict[i]['curcio_derbyshire_equiv'],
            piegu_equiv=file_info_dict[i]['piegu_equiv']
        )

        clade_dict[tc.lineage] = tc

    # add clade relationship

    for i in clade_dict:
        tc = clade_dict[i]

        if tc.parent:
            parent_lineage = tc.lineage[:-1]
            p_tc = clade_dict[parent_lineage]
            if p_tc.clades is None:
                p_tc.clades = []
            p_tc.clades.append(tc)

    top_clade = [clade_dict[i]
                 for i in clade_dict if clade_dict[i].parent is None]
    for i in top_clade:
        i.parent = 'root'

    root_clade = Clade(name='root', clades=top_clade)

    te_class_tree = Tree(root_clade, True)

    return te_class_tree


def parse_dfam_hmm_file(dfam_hmm_file):

    f = open(dfam_hmm_file, 'r')

    record_info_dict = {}

    for each_line in f:
        each_line = re.sub(r'\n', '', each_line)

        if re.match(r'^#', each_line):
            continue

        # skip HMM
        match = re.match(r'^ .*', each_line)
        if match:
            continue

        match_list = re.findall(r'^(\S+)\s+(.*)$', each_line)
        if len(match_list) > 0:
            match_list = match_list[0]
            k, v = match_list
            if k not in record_info_dict:
                record_info_dict[k] = []
            record_info_dict[k].append(v)

        if each_line == "//":
            yield record_info_dict
            record_info_dict = {}

    f.close()

    if len(record_info_dict) > 0:
        yield record_info_dict


def load_dfam_info_to_sqlite(dfam_hmm_file, db_name, log_file=None):
    module_log = logging_init("load_dfam_info_to_sqlite", log_file)
    module_log.info(
        'received a call to "lib.resource.dfam.load_dfam_info_to_sqlite"')

    # making a new sql database for store info'
    table_columns_dict = {
        "record": ["acc", "pickle_string"]
    }
    sc.init_sql_db_many_table(db_name, table_columns_dict)
    module_log.info('made a new sql database for store info')

    # loading fasta file and store to sqlite
    module_log.info('loading dfam hmm file and storing in sqlite')
    start_time = time.time()
    num = 0
    record_tmp_list = []
    for record in parse_dfam_hmm_file(dfam_hmm_file):
        accession = record['ACC'][0]
        record_pickle = pickle.dumps(record)
        record_tmp_list.append((accession, record_pickle))

        num = num + 1

        if num % 10000 == 0:
            sc.sqlite_write(record_tmp_list, db_name, "record",
                            table_columns_dict["record"])
            record_tmp_list = []

        round_time = time.time()
        if round_time - start_time > 10:
            module_log.info("\tparsed: %d" % (num))
            start_time = round_time

    if len(record_tmp_list) > 0:
        sc.sqlite_write(record_tmp_list, db_name, "record",
                        table_columns_dict["record"])
        record_tmp_list = []
        module_log.info("\tparsed: %d" % (num))

    module_log.info('loaded dfam file and stored in sqlite ')

    conn = sqlite3.connect(db_name)
    conn.execute("CREATE UNIQUE INDEX access_index on %s (\"%s\")" %
                 ("record", "acc"))
    conn.close()

    del module_log.handlers[:]


def get_dfam_info(dfam_accession_list, db_name):
    db_out_list = sc.sqlite_select(
        db_name, 'record', key_name='acc', value_tuple=tuple(dfam_accession_list))
    output_dict = {}
    for acc, info_pickle in db_out_list:
        info = pickle.loads(info_pickle)
        output_dict[acc] = info
    return output_dict


def get_seq_anntation(dfamtblout, db_name, dfam_TE_classes_file=None, e_value_thr=1e-5, cover=0.6):
    """
    nhmmscan --noali --dfamtblout new_9.dfamtblout --cpu=10 /lustre/home/xuyuxing/Database/Dfam/Dfam.hmm new_9.fa
    """
    hmm_out_dict = {}
    with open(dfamtblout, 'r') as f:
        for each_line in f:
            each_line = re.sub(r'\n', '', each_line)
            if re.match(r'^#', each_line):
                continue

            info = each_line.split()
            target_name, acc, q_name, bits, e_value, bias, hmm_st, hmm_en, strand, ali_st, ali_en, env_st, env_en, modlen = info[
                0:14]
            target_name, acc, q_name, bits, e_value, bias, hmm_st, hmm_en, strand, ali_st, ali_en, env_st, env_en, modlen = target_name, acc, q_name, float(
                bits), float(e_value), float(bias), int(hmm_st), int(hmm_en), strand, int(ali_st), int(ali_en), int(env_st), int(env_en), int(modlen)

            if e_value <= e_value_thr and (abs(env_en - env_st) + 1 / modlen) >= cover:
                if q_name not in hmm_out_dict:
                    hmm_out_dict[q_name] = []
                hmm_out_dict[q_name].append((target_name, acc, q_name, bits, e_value,
                                             bias, hmm_st, hmm_en, strand, ali_st, ali_en, env_st, env_en, modlen))

    dfam_acc_list = []
    for i in hmm_out_dict:
        for j in hmm_out_dict[i]:
            dfam_acc_list.append(j[1])

    dfam_acc_list = list(set(dfam_acc_list))
    dfam_acc_info = get_dfam_info(dfam_acc_list, db_name)

    dfam_lineage_dict = {}
    for i in dfam_acc_info:
        lineage = tuple(dfam_acc_info[i]['CT'][0].split(";"))

        # if 'Retrotransposed_Element' in lineage:
        #     lineage_new = []
        #     for j in lineage:
        #         if j == 'Retrotransposed_Element':
        #             lineage_new.append('Class_I_Retrotransposition')
        #         else:
        #             lineage_new.append(j)
        #     lineage = tuple(lineage_new)

        # if 'DNA_Transposon' in lineage:
        #     lineage_new = []
        #     for j in lineage:
        #         if j == 'DNA_Transposon':
        #             lineage_new.append('Class_II_DNA_Transposition')
        #         else:
        #             lineage_new.append(j)
        #     lineage = tuple(lineage_new)

        dfam_lineage_dict[i] = lineage

    te_tree = build_TE_class_tree()

    te_dict = {}
    for clade in te_tree.find_clades(order='preorder'):
        if hasattr(clade, "lineage"):
            te_dict[clade.lineage] = clade
            # print(clade.lineage)

    for i in dfam_lineage_dict:
        j = dfam_lineage_dict[i]
        if not j in te_dict:
            raise ValueError("unknown dfam linage: %s" % ";".join(j))

    #
    query_lineage_dict = {}
    for q_name in hmm_out_dict:
        if len(hmm_out_dict[q_name]) > 0:
            for i in hmm_out_dict[q_name]:
                dfam_acc = i[1]
                if dfam_acc in dfam_lineage_dict:
                    lineage = dfam_lineage_dict[dfam_acc]
                    if len(lineage) > 0:
                        query_lineage_dict[q_name] = lineage
                        break

    return query_lineage_dict


if __name__ == '__main__':
    dfam_hmm_file = "/lustre/home/xuyuxing/Database/Dfam/Dfam.hmm"
    db_name = "/lustre/home/xuyuxing/Database/Dfam/Dfam.info.db"
    log_file = "/lustre/home/xuyuxing/Database/Dfam/log"

    load_dfam_info_to_sqlite(dfam_hmm_file, db_name, log_file=log_file)



import os
from lib.common.util import logging_init
from lib.common.os import cmd_run, mkdir
from api.xuyuxing.genome.maker_config_file_code import write_maker_config, write_maker_evaluate_anno_shell, write_maker_to_snap_augustus_shell, write_inherit_maker_gff_shell


class Maker_Job(object):
    def __init__(self, args):
        # parse argument
        self.args = args

    def first_round(self, work_dir):
        mkdir(work_dir, True)
        config_file = work_dir + "/maker_config.txt"

        genome_file = work_dir+"/genome.fasta"
        os.symlink(os.path.abspath(self.args.genome_file), genome_file)

        repeat_file = work_dir+"/repeat.fasta"
        os.symlink(os.path.abspath(self.args.repeat_file), repeat_file)

        est_file = work_dir+"/est.fasta"
        os.symlink(os.path.abspath(self.args.est_file), est_file)

        protein_file = work_dir+"/protein.fasta"
        os.symlink(os.path.abspath(self.args.protein_file), protein_file)

        rRNA_file = work_dir+"/rRNA.fasta"
        os.symlink(os.path.abspath(self.args.rrna_file), rRNA_file)

        repeat_protein = work_dir+"/repeat_protein.fasta"
        os.symlink(os.path.abspath(self.args.repeat_protein), repeat_protein)

        TMP_dir = work_dir+"/TMP"
        mkdir(TMP_dir)

        write_maker_config(
            config_file=config_file,
            genome=genome_file,
            est=est_file,
            protein=protein_file,
            rmlib=repeat_file,
            repeat_protein=repeat_protein,
            augustus_species=self.args.augustus_species,
            est2genome=1,
            protein2genome=1,
            snoscan_rrna=rRNA_file,
            TMP=TMP_dir,
        )

        print("Please running command:")
        print("cd %s" % work_dir)
        print("source activate maker_p")
        print("nohup mpiexec -n 56 maker --ignore_nfs_tmp maker_config.txt > nohup.txt &")

    def evaluate_anno(self, maker_output_dir):
        """
        maker_output_dir = "/lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/maker/maker_work/genome.maker.output"
        """

        evaluate_anno_shell_file = maker_output_dir + "/evaluate_anno.sh"

        index_log_file = maker_output_dir + '/genome_master_datastore_index.log'

        write_maker_evaluate_anno_shell(
            evaluate_anno_shell_file, index_log_file)

        cmd_run_string = "bash " + evaluate_anno_shell_file

        cmd_run(cmd_run_string, cwd=maker_output_dir,
                retry_max=1, silence=True, log_file=None)

    def to_next_round(self, maker_output_dir):
        """
        maker_output_dir = "/lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/maker/maker_work/genome.maker.output"
        """
        to_next_dir = maker_output_dir + "/to_next"
        mkdir(to_next_dir, True)
        maker_to_snap_augustus_shell_file = to_next_dir + "/maker_to_snap_augustus.sh"
        index_log_file = maker_output_dir + '/genome_master_datastore_index.log'

        # train snap and augustus
        write_maker_to_snap_augustus_shell(maker_to_snap_augustus_shell_file)

        cmd_run_string = "bash %s %s %s" % (
            maker_to_snap_augustus_shell_file, self.args.tag, index_log_file)

        cmd_run(cmd_run_string, cwd=to_next_dir,
                retry_max=1, silence=True, log_file=None)

        # rewrite gff to next round
        inherit_maker_gff_shell_file = maker_output_dir + "/inherit_maker_gff.sh"
        write_inherit_maker_gff_shell(inherit_maker_gff_shell_file)

        cmd_run_string = "bash " + inherit_maker_gff_shell_file

        cmd_run(cmd_run_string, cwd=maker_output_dir,
                retry_max=1, silence=True, log_file=None)

        print("est_gff: %s/for_next.est2genome.gff" % maker_output_dir)
        print("protein_gff: %s/for_next.protein2genome.gff" % maker_output_dir)
        print("rm_gff: %s/for_next.repeats.gff" % maker_output_dir)
        print("snaphmm: %s/to_next/%s.zff.length50_aed0.25.hmm" %
              (maker_output_dir, self.args.tag))
        print("augustus_species: %s" % self.args.tag)

    def improve_round(self, work_dir):
        mkdir(work_dir, True)
        config_file = work_dir + "/maker_config.txt"

        genome_file = work_dir+"/genome.fasta"
        os.symlink(os.path.abspath(self.args.genome_file), genome_file)

        rm_gff = work_dir+"/rm.gff"
        os.symlink(os.path.abspath(self.args.rm_gff), rm_gff)

        est_gff = work_dir+"/est.gff"
        os.symlink(os.path.abspath(self.args.est_gff), est_gff)

        protein_gff = work_dir+"/protein.gff"
        os.symlink(os.path.abspath(self.args.protein_gff), protein_gff)

        snaphmm = work_dir+"/snaphmm.hmm"
        os.symlink(os.path.abspath(self.args.snaphmm), snaphmm)

        rRNA_file = work_dir+"/rRNA.fasta"
        os.symlink(os.path.abspath(self.args.rrna_file), rRNA_file)

        TMP_dir = work_dir+"/TMP"
        mkdir(TMP_dir)

        write_maker_config(
            config_file=config_file,
            genome=genome_file,
            est_gff=est_gff,
            protein_gff=protein_gff,
            rm_gff=rm_gff,
            snaphmm=snaphmm,
            augustus_species=self.args.augustus_species,
            est2genome=0,
            protein2genome=0,
            snoscan_rrna=rRNA_file,
            TMP=TMP_dir,
        )

        print("Please running command:")
        print("cd %s" % work_dir)
        print("source activate maker_p")
        print("nohup mpiexec -n 56 maker --ignore_nfs_tmp maker_config.txt > nohup.txt &")


def maker_main(args):
    mkdir(args.work_dir, True)
    log_file = args.work_dir + "/log.txt"

    logger = logging_init("maker main", log_file)

    args_info_string = "Argument list:\n"
    attrs = vars(args)

    for item in attrs.items():
        args_info_string = args_info_string + ("%s: %s\n" % item)

    logger.info(args_info_string)

    maker_job = Maker_Job(args)

    if args.job == 'first_anno':
        # first round
        logger.info("Running first_anno at %s" % args.work_dir)
        maker_job.first_round(args.work_dir)

    elif args.job == 'evaluate_anno':
        # evaluate_anno
        logger.info("Running evaluate_anno at %s" % args.work_dir)
        maker_job.evaluate_anno(args.work_dir)

    elif args.job == 'to_next_round':
        # evaluate_anno
        logger.info("Running to_next_round at %s" % args.work_dir)
        maker_job.to_next_round(args.work_dir)

    elif args.job == 'improve_anno':
        # evaluate_anno
        logger.info("Running improve_anno at %s" % args.work_dir)
        maker_job.improve_round(args.work_dir)


if __name__ == '__main__':
    # first round
    # need a configure file

    """
    [Parameter]
    num_threads = 56

    [Paths]
    tag = Llv
    genome_file = /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/Llv.genome.nt.1.0.fasta
    repeat_file = /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/maker_p/Llv_maker_p/allRepeats.lib
    est_file = /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/trinity/trinity_output/Trinity.fasta
    protein_file = /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/close.aa.fasta
    rRNA_file = /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/rRNA/rRNA.no_ITS.fa
    augustus_species = tomato
    repeat_protein = /lustre/home/xuyuxing/Database/Phelipanche/annotation/maker_p/repeat/Tpase/Tpases020812
    """

    # python ~/python_project/Genome_work_tools/GenAnno.py EasyMaker -j first_anno -c maker.config -d /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/maker
    # nohup mpiexec -n 56 maker --ignore_nfs_tmp maker.config > nohup.txt &

    # python ~/python_project/Genome_work_tools/GenAnno.py EasyMaker -d /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/maker/maker_work/genome.maker.output -j evaluate_anno
    # python ~/python_project/Genome_work_tools/GenAnno.py EasyMaker -d /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/maker/maker_work/genome.maker.output -j to_next_round -t Llv_v1

    """
    [Parameter]
    num_threads = 56

    [Paths]
    tag = Llv
    genome_file = /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/Llv.genome.nt.1.0.fasta
    est_gff = /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/maker/maker_work/genome.maker.output/for_next.est2genome.gff
    protein_gff = /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/maker/maker_work/genome.maker.output/for_next.protein2genome.gff
    rm_gff = /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/maker/maker_work/genome.maker.output/for_next.repeats.gff
    snaphmm = /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/maker/maker_work/genome.maker.output/to_next/Llv_v1.zff.length50_aed0.25.hmm
    augustus_species = Llv_v1
    rRNA_file = /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/rRNA/rRNA.no_ITS.fa
    """

    # python ~/python_project/Genome_work_tools/GenAnno.py EasyMaker -j improve_anno -c maker_r2.config -d /lustre/home/xuyuxing/Database/Lindenbergia/genome/annotation/maker/round2
    # nohup mpiexec -n 56 maker --ignore_nfs_tmp maker.config > nohup.txt &



if __name__ == '__main__':
    import argparse

    # argument parse
    parser = argparse.ArgumentParser(
        prog='OrthoTools',
    )

    subparsers = parser.add_subparsers(
        title='subcommands', dest="subcommand_name")

    # argparse for EasyHcluster
    parser_a = subparsers.add_parser('EasyHcluster',
                                     help='Easy tool for Hcluster pipeline')
    parser_a.add_argument(
        "proteins_dir", help="just like orthofinder input, make all species protein in one dir", type=str)
    parser_a.add_argument("-o", "--work_dir",
                          help="output work dir", type=str, default="hcluster")
    parser_a.add_argument("-t", "--num_threads",
                          help="output file", type=int, default=56)
    parser_a.add_argument("-op", "--option_string",
                          help="option for hcluster, defaults: -m 750 -w 0 -s 0.34 -O", type=str, default="-m 750 -w 0 -s 0.34 -O")

    # argparse for treebestAllGO
    parser_a = subparsers.add_parser('treebestAllGO',
                                     help='TreeBeST pipeline from a orthogroups file')
    parser_a.add_argument(
        "OG_tsv_file", help="orthofinder orthogroups tsv file", type=str)
    parser_a.add_argument(
        "species_info_table", help="species info table excel file, should have: sp_id, OG_species_id, pt_file, cds_file", type=str)
    parser_a.add_argument("SpeciesTree_rooted_file",
                          help="can use orthofinder SpeciesTree_rooted.txt file", type=str)
    parser_a.add_argument("-o", "--work_dir",
                          help="output work dir", type=str, default=".")
    parser_a.add_argument("-t", "--num_threads",
                          help="num_threads", type=int, default=10)

    # argparse for fasttreeAllGO
    parser_a = subparsers.add_parser('fasttreeAllGO',
                                     help='get fasttree for an orthogroups file')
    parser_a.add_argument(
        "OG_tsv_file", help="orthofinder orthogroups tsv file", type=str)
    parser_a.add_argument(
        "species_info_table", help="species info table excel file, should have: sp_id, OG_species_id, pt_file", type=str)
    parser_a.add_argument("-s", "--species_tree",
                          help="can use orthofinder SpeciesTree_rooted.txt file", type=str)
    parser_a.add_argument("-o", "--work_dir",
                          help="output work dir", type=str, default=".")
    parser_a.add_argument("-t", "--num_threads",
                          help="num_threads", type=int, default=10)

    # argparse for GetOrthogroupsFromTreebest
    parser_a = subparsers.add_parser('GetOrthogroupsFromTreebest',
                                     help='get orthologous tsv file from TreeBeST with a given taxonomy level')
    parser_a.add_argument(
        "treebest_work_dir", help="path of treebest work dir", type=str)
    parser_a.add_argument("SpeciesTree_rooted_file",
                          help="can use orthofinder SpeciesTree_rooted.txt file", type=str)
    parser_a.add_argument(
        "taxonomy_level", help="taxonomy level in species tree", type=str)
    parser_a.add_argument(
        "-o", "--output_file", help="path of output file", type=str, default="orthologous.tsv")
    parser_a.add_argument(
        "-k", "--used_OG_list_file", help="give me a OG id list file to use, and skip other OG, if not have file will use all OG in treebest dir", type=str)

    # argparse for GetOrthogenesFromTreebest
    parser_a = subparsers.add_parser('GetOrthogenesFromTreebest',
                                     help='get species pairs orthogenes tsv file from TreeBeST with a given species pair, its will be more good for gene loss, it based on most ')
    parser_a.add_argument(
        "treebest_work_dir", help="path of treebest work dir", type=str)
    parser_a.add_argument("SpeciesTree_rooted_file",
                          help="can use orthofinder SpeciesTree_rooted.txt file", type=str)
    parser_a.add_argument(
        "-o", "--output_file", help="path of output file", type=str, default="orthologous.tsv")

    # argparse for OrthofinderGroup
    parser_a = subparsers.add_parser('OrthofinderGroup',
                                     help='get orthogroups from Orthofinder output')
    parser_a.add_argument(
        "orthologues_dir", help="Path of Orthologues dir from orthofinder output", type=str)
    parser_a.add_argument("output_file", help="Path of output file", type=str)

    # argparse for Orthologues pair
    parser_a = subparsers.add_parser('OrthoPair',
                                     help='Orthologues pair')
    parser_a.add_argument(
        "orthologues_tsv", help="orthologues tsv file", type=str)
    parser_a.add_argument("species1", help="species1", type=str)
    parser_a.add_argument("-sp2", "--species2", help="species2", type=str)
    parser_a.add_argument("output_file", help="output file", type=str)

    # argparse for mcscanxGO
    parser_a = subparsers.add_parser('mcscanxGO',
                                     help='Running MCScanX')
    parser_a.add_argument("sp1_gff", help="species1 gff3 file", type=str)
    parser_a.add_argument("-sp2_gff", "--sp2_gff",
                          help="species2 gff3 file", type=str)
    parser_a.add_argument("-sp1", "--sp1_prefix",
                          help="species 1 prefix in mcscanx running", default="aa", type=str)
    parser_a.add_argument("-sp2", "--sp2_prefix",
                          help="species 2 prefix in mcscanx running", default="bb", type=str)
    parser_a.add_argument("-sp1_aa", "--sp1_aa_fasta",
                          help="species 1 protein fasta file", type=str)
    parser_a.add_argument("-sp2_aa", "--sp2_aa_fasta",
                          help="species 2 protein fasta file", type=str)
    parser_a.add_argument("-bls", "--blast_results_file",
                          help="blast results file", type=str)
    parser_a.add_argument("-s", "--skip_gene_list_file",
                          help="skip gene list file", type=str)
    parser_a.add_argument("-w", "--work_dir",
                          help="work dir defaults .", default=".", type=str)
    parser_a.add_argument("-o", "--mcscanx_options",
                          help="mcscanx options defaults: '-k 50 -g -1 -s 5 -e 1e-30 -m 25' ", default="-k 50 -g -1 -s 5 -e 1e-30 -m 25", type=str)
    parser_a.add_argument("-c", "--c_score_threshold",
                          help="c_score_threshold defaults: 0", default=0, type=float)

    # argparse for mcscanxhGO
    parser_a = subparsers.add_parser('mcscanxhGO',
                                     help='Running MCScanX_h')
    parser_a.add_argument("sp1_gff", help="species1 gff3 file", type=str)
    parser_a.add_argument(
        "orthologues_tsv", help="orthologues tsv file", type=str)
    parser_a.add_argument("-sp2_gff", "--sp2_gff",
                          help="species2 gff3 file", type=str)
    parser_a.add_argument("-sp1", "--sp1_prefix",
                          help="species 1 prefix in mcscanx running", default="aa", type=str)
    parser_a.add_argument("-sp2", "--sp2_prefix",
                          help="species 2 prefix in mcscanx running", default="bb", type=str)
    parser_a.add_argument("-w", "--work_dir",
                          help="work dir defaults .", default=".", type=str)
    parser_a.add_argument("-f", "--huge_gene_family_filter",
                          help="filter huge gene family, for self comparion", default=50, type=int)
    parser_a.add_argument("-o", "--mcscanx_options",
                          help="mcscanx options defaults: '-k 30 -g 0 -s 5 -e 1e-05 -m 25' ", default="-k 30 -g 0 -s 5 -e 1e-05 -m 25", type=str)

    # argparse for ConvertCollinearity
    parser_a = subparsers.add_parser('ConvertCollinearity',
                                     help='Convert Collinearity File to csv')
    parser_a.add_argument("query_id", help="query id", type=str)
    parser_a.add_argument("query_gff", help="query gff file", type=str)
    parser_a.add_argument("subject_id", help="subject id", type=str)
    parser_a.add_argument("subject_gff", help="subject gff file", type=str)
    parser_a.add_argument("mcscanx_collinearity_file",
                          help="mcscanx_collinearity_file", type=str)
    parser_a.add_argument("csv_file", help="csv_file", type=str)

    # argparse for countWGD
    parser_a = subparsers.add_parser('countWGD',
                                     help='count WGD from mcscanx.collinearity')
    parser_a.add_argument("sp1", help="sp1_prefix", type=str)
    parser_a.add_argument("sp2", help="sp2_prefix", type=str)
    parser_a.add_argument("sp1_gff", help="species1 gff3 file", type=str)
    parser_a.add_argument("sp2_gff", help="species2 gff3 file", type=str)
    parser_a.add_argument("mcscanx_collinearity_file",
                          help="path of mcscanx_collinearity_file", type=str)
    parser_a.add_argument("-o", "--OG_tsv_file",
                          help="path of OG_tsv_file", type=str)
    parser_a.add_argument("-t", "--OG_filter",
                          help="OG_filter threshold", default=0.3, type=float)

    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    # command detail

    if args_dict["subcommand_name"] == "OrthofinderGroup":
        # pre
        """
        class abc(object):
            pass

        args = abc()
        args.orthologues_dir = '/lustre/home/xuyuxing/Work/Gel/mcscanx/Gel_vs_Aof/orthofinder2/OrthoFinder/Results_Nov21/Orthologues'
        args.output_file = '/lustre/home/xuyuxing/Work/Gel/mcscanx/Gel_vs_Aof/orthofinder2/OrthoFinder/Results_Nov21/Orthologues/Orthologues.tsv'
        """

        from api.xuyuxing.comp_genome.orthofinder import get_orthologues
        get_orthologues(args.orthologues_dir, args.output_file)

    elif args_dict["subcommand_name"] == "OrthoPair":
        from api.xuyuxing.comp_genome.orthofinder import tsv_to_gene_pair
        tsv_to_gene_pair(args.orthologues_tsv, args.species1,
                         args.species2, args.output_file)

    elif args_dict["subcommand_name"] == "mcscanxGO":

        from api.xuyuxing.comp_genome.mcscan import running_mcscanx
        from lib.common.fileIO import read_list_file
        import os

        work_dir = os.path.abspath(args.work_dir)
        if args.skip_gene_list_file:
            skip_gene_list = read_list_file(args.skip_gene_list_file)
        else:
            skip_gene_list = []

        if not args.sp2_gff is None:
            running_mcscanx(args.sp1_gff, args.sp2_gff, work_dir, species1_prefix=args.sp1_prefix, species1_aa_fasta=args.sp1_aa_fasta,
                            species2_prefix=args.sp2_prefix, species2_aa_fasta=args.sp2_aa_fasta, skip_gene_list=skip_gene_list, give_bls_file=args.blast_results_file, mcscanx_options=args.mcscanx_options, c_score_threshold=args.c_score_threshold)
        else:
            running_mcscanx(args.sp1_gff, None, work_dir, species1_prefix=args.sp1_prefix, species1_aa_fasta=args.sp1_aa_fasta,
                            species2_prefix=None, species2_aa_fasta=None, skip_gene_list=skip_gene_list, give_bls_file=args.blast_results_file, mcscanx_options=args.mcscanx_options + " -b 1", c_score_threshold=args.c_score_threshold)

    elif args_dict["subcommand_name"] == "mcscanxhGO":

        from api.xuyuxing.comp_genome.mcscan import running_mcscanxh
        import os

        work_dir = os.path.abspath(args.work_dir)

        if not args.sp2_gff is None:
            running_mcscanxh(args.sp1_gff, args.sp2_gff, args.orthologues_tsv, work_dir,
                             species1_prefix=args.sp1_prefix, species2_prefix=args.sp2_prefix, mcscanx_options=args.mcscanx_options)
        else:
            running_mcscanxh(args.sp1_gff, None, args.orthologues_tsv, work_dir, species1_prefix=args.sp1_prefix,
                             species2_prefix=None, mcscanx_options=args.mcscanx_options + " -b 1", huge_gene_family_filter=args.huge_gene_family_filter)

    elif args_dict["subcommand_name"] == "treebestGO":
        pass

    elif args_dict["subcommand_name"] == "treebestAllGO":
        from api.xuyuxing.comp_genome.treebest import treebest_many
        treebest_many(args.OG_tsv_file, args.species_info_table,
                      args.SpeciesTree_rooted_file, args.work_dir, args.num_threads)

    elif args_dict["subcommand_name"] == "fasttreeAllGO":
        from api.xuyuxing.comp_genome.fasttree import fasttree_many
        from lib.common.evolution.orthotools2 import OrthoGroups, read_species_info
        from lib.common.evolution.tree_operate import get_root_by_species, add_clade_name, lookup_by_names
        from Bio import Phylo

        OGs = OrthoGroups(OG_tsv_file=args.OG_tsv_file)
        species_info_dict = read_species_info(args.species_info_table)

        fasttree_many(OGs, species_info_dict, args.work_dir, args.num_threads)

        if args.species_tree:
            species_tree = Phylo.read(args.species_tree, 'newick')
            species_tree = add_clade_name(species_tree)
            species_tree_dict = lookup_by_names(species_tree)

        get_root_by_species

        get_root_by_species(gene_tree, species_rooted_tree,
                            gene_to_species_map)

    elif args_dict["subcommand_name"] == "countWGD":
        from api.xuyuxing.comp_genome.mcscan import WGD_check_pipeline
        from collections import Counter

        if args.sp1 == sorted([args.sp1, args.sp2])[0]:
            q = args.sp1
            s = args.sp2
            q_gff = args.sp1_gff
            s_gff = args.sp2_gff
        else:
            q = args.sp2
            s = args.sp1
            q_gff = args.sp2_gff
            s_gff = args.sp1_gff

        q_gene_covered_dict, q_range_loci_cover_chr_dict, q_range_base_cover_chr_dict, s_gene_covered_dict, s_range_loci_cover_chr_dict, s_range_base_cover_chr_dict, synteny_block_dict = WGD_check_pipeline(
            args.mcscanx_collinearity_file, q, q_gff, None, s, s_gff, None)

        print(args.sp2+": ")
        print(Counter([q_gene_covered_dict[i] for i in q_gene_covered_dict]))

        print(args.sp1+": ")
        print(Counter([s_gene_covered_dict[i] for i in s_gene_covered_dict]))

    elif args_dict["subcommand_name"] == "GetOrthogroupsFromTreebest":
        from api.xuyuxing.comp_genome.treebest import get_orthologous_from_treebest
        from lib.common.evolution.orthotools2 import write_OG_tsv_file
        from lib.common.fileIO import read_list_file

        if args.used_OG_list_file:
            used_OG_list = read_list_file(args.used_OG_list_file)
        else:
            used_OG_list = None

        OL_dict = get_orthologous_from_treebest(
            args.treebest_work_dir, args.SpeciesTree_rooted_file, taxonomy_level=args.taxonomy_level, used_OG_list=used_OG_list)

        write_OG_tsv_file(OL_dict, args.output_file)

    elif args_dict["subcommand_name"] == "ConvertCollinearity":
        from api.xuyuxing.comp_genome.mcscan import collinearity_file_to_synteny_block_range_csv
        collinearity_file_to_synteny_block_range_csv(
            args.query_id, args.query_gff, args.subject_id, args.subject_gff, args.mcscanx_collinearity_file, args.csv_file)

    elif args_dict["subcommand_name"] == "EasyHcluster":
        from src.xuyuxing.OrthoTools.EasyHcluster import hcluster_pipeline
        hcluster_pipeline(args.proteins_dir, args.work_dir,
                          args.num_threads, args.option_string)
